#include "canvas.h"

#include <QtGui>
#include "abstractobj.h"
#include "objtypemimedata.h"
#include "model.h"
#include "objectlistproxymodel.h"

// Objects for canvas
#include "buttonforcanvas.h"
#include "spinnerforcanvas.h"
#include "checkboxforcanvas.h"
#include "radioforcanvas.h"
#include "groupboxforcanvas.h"
#include "editforcanvas.h"
#include "varlistforcanvas.h"
#include "varnameforcanvas.h"
#include "textforcanvas.h"
#include "textboxforcanvas.h"
#include "frameforcanvas.h"
#include "colorforcanvas.h"
#include "expforcanvas.h"
#include "comboboxforcanvas.h"
#include "fileforcanvas.h"
#include "listboxforcanvas.h"

// Objects
#include "buttonobj.h"
#include "spinnerobj.h"
#include "checkboxobj.h"
#include "radioobj.h"
#include "groupboxobj.h"
#include "editobj.h"
#include "varlistobj.h"
#include "varnameobj.h"
#include "textobj.h"
#include "textboxobj.h"
#include "frameobj.h"
#include "colorobj.h"
#include "expobj.h"
#include "comboboxobj.h"
#include "fileobj.h"
#include "listboxobj.h"

#include <QDebug>

Canvas::Canvas(QWidget *parent)
    : QAbstractItemView(parent), idealWidth(0), idealHeight(0), hashIsDirty(false)
{
    rowOfSelectedDialog = -1;
    setFocusPolicy(Qt::WheelFocus);
    horizontalScrollBar()->setRange(0, 0);
    verticalScrollBar()->setRange(0, 0);
    setAcceptDrops(true);
    setDropIndicatorShown(true);

    // Font
    stataDialogFont.setFamily("Microsoft Sans Serif");
    stataDialogFont.setPixelSize(11);
}

void Canvas::setModel(QAbstractItemModel *model)
{
    QAbstractItemView::setModel(model);
    hashIsDirty = true;
}

void Canvas::calculateRectsIfNecessary() const
{
    if (!hashIsDirty) return;

    objsForCanvas.clear();
    QRect previousRect(-1, -1, -1, -1);
    int nRows = model()->rowCount();
    for (int row=0; row<nRows; row++)
    {
        QModelIndex mainIndex = model()->index(row, 0, QModelIndex());
        AbstractObj *mainObj = static_cast<AbstractObj*>(mainIndex.internalPointer());
        if (mainObj->objType() == AbstractObj::Dialog)
        {
            ObjsForCanvasInDialog objsForCanvasInDialog;
            objsForCanvas.insert(mainIndex.row(), objsForCanvasInDialog);
            int nObjsInDialog = model()->rowCount(mainIndex);
            for (int subRow=0; subRow<nObjsInDialog; subRow++)
            {
                QModelIndex subIndex = model()->index(subRow, 0, mainIndex);
                AbstractObj *subObj = static_cast<AbstractObj*>(subIndex.internalPointer());
                switch (subObj->objType())
                {
                    case AbstractObj::Button:
                        {
                        ButtonObj *buttonObj = static_cast<ButtonObj*>(subObj);
                        QRect rect = rectForControlObj(buttonObj, previousRect);
                        ButtonForCanvas *buttonForCanvas
                                = new ButtonForCanvas(rect, buttonObj->labelProperty->value(), this);
                        objsForCanvas[mainIndex.row()].objs.insert(subRow, buttonForCanvas);
                        previousRect = rect;
                        }
                        break;
                    case AbstractObj::Spinner:
                        {
                        SpinnerObj *spinnerObj = static_cast<SpinnerObj*>(subObj);
                        QRect rect = rectForControlObj(spinnerObj, previousRect);
                        SpinnerForCanvas *spinnerForCanvas
                                = new SpinnerForCanvas(rect, this);
                        objsForCanvas[mainIndex.row()].objs.insert(subRow, spinnerForCanvas);
                        previousRect = rect;
                        }
                        break;
                    case AbstractObj::CheckBox:
                        {
                        CheckBoxObj *checkBoxObj = static_cast<CheckBoxObj*>(subObj);
                        QRect rect = rectForControlObj(checkBoxObj, previousRect);
                        CheckBoxForCanvas *checkBoxForCanvas
                                = new CheckBoxForCanvas(rect, checkBoxObj->labelProperty->value(), this);
                        objsForCanvas[mainIndex.row()].objs.insert(subRow, checkBoxForCanvas);
                        previousRect = rect;
                        }
                        break;
                    case AbstractObj::Radio:
                        {
                        RadioObj *radioObj = static_cast<RadioObj*>(subObj);
                        QRect rect = rectForControlObj(radioObj, previousRect);
                        RadioForCanvas *radioForCanvas
                                = new RadioForCanvas(rect, radioObj->labelProperty->value(), this);
                        objsForCanvas[mainIndex.row()].objs.insert(subRow, radioForCanvas);
                        previousRect = rect;
                        }
                        break;
                    case AbstractObj::GroupBox:
                        {
                        GroupBoxObj *groupBoxobj = static_cast<GroupBoxObj*>(subObj);
                        QRect rect = rectForControlObj(groupBoxobj, previousRect);
                        GroupBoxForCanvas *groupBoxForCanvas
                                = new GroupBoxForCanvas(rect, groupBoxobj->labelProperty->value(), this);
                        objsForCanvas[mainIndex.row()].objs.insert(subRow, groupBoxForCanvas);
                        previousRect = rect;
                        }
                        break;
                    case AbstractObj::Edit:
                        {
                        EditObj *editObj = static_cast<EditObj*>(subObj);
                        QRect rect = rectForControlObj(editObj, previousRect);
                        EditForCanvas *editForCanvas
                                = new EditForCanvas(rect, this);
                        objsForCanvas[mainIndex.row()].objs.insert(subRow, editForCanvas);
                        previousRect = rect;
                        }
                        break;
                    case AbstractObj::VarList:
                        {
                        VarListObj *varListObj = static_cast<VarListObj*>(subObj);
                        QRect rect = rectForControlObj(varListObj, previousRect);
                        VarListForCanvas *varListForCanvas
                                = new VarListForCanvas(rect, this);
                        objsForCanvas[mainIndex.row()].objs.insert(subRow, varListForCanvas);
                        previousRect = rect;
                        }
                        break;
                    case AbstractObj::VarName:
                        {
                        VarNameObj *varNameObj = static_cast<VarNameObj*>(subObj);
                        QRect rect = rectForControlObj(varNameObj, previousRect);
                        VarNameForCanvas *varNameForCanvas
                                = new VarNameForCanvas(rect, this);
                        objsForCanvas[mainIndex.row()].objs.insert(subRow, varNameForCanvas);
                        previousRect = rect;
                        }
                        break;
                    case AbstractObj::Text:
                        {
                        TextObj *textObj = static_cast<TextObj*>(subObj);
                        QString alignmentValue = textObj->alignmentProperty->value();
                        Qt::Alignment alignment;
                        if (alignmentValue == "center")
                            alignment = Qt::AlignCenter;
                        else if (alignmentValue == "right")
                            alignment = Qt::AlignRight;
                        else
                            alignment = Qt::AlignLeft;
                        alignment |= Qt::AlignVCenter;
                        QRect rect = rectForControlObj(textObj, previousRect);
                        TextForCanvas *textForCanvas
                                = new TextForCanvas(rect, textObj->labelProperty->value(),
                                                    alignment, this);
                        objsForCanvas[mainIndex.row()].objs.insert(subRow, textForCanvas);
                        previousRect = rect;
                        }
                        break;
                    case AbstractObj::TextBox:
                        {
                        TextBoxObj *textBoxObj = static_cast<TextBoxObj*>(subObj);
                        QString alignmentValue = textBoxObj->alignmentProperty->value();
                        Qt::Alignment alignment;
                        if (alignmentValue == "center")
                            alignment = Qt::AlignCenter;
                        else if (alignmentValue == "right")
                            alignment = Qt::AlignRight;
                        else
                            alignment = Qt::AlignLeft;
                        QRect rect = rectForControlObj(textBoxObj, previousRect);
                        TextBoxForCanvas *textBoxForCanvas
                                = new TextBoxForCanvas(rect, textBoxObj->labelProperty->value(),
                                                    alignment, this);
                        objsForCanvas[mainIndex.row()].objs.insert(subRow, textBoxForCanvas);
                        previousRect = rect;
                        }
                        break;
                    case AbstractObj::Frame:
                        {
                        FrameObj *frameObj = static_cast<FrameObj*>(subObj);
                        QRect rect = rectForControlObj(frameObj, previousRect);
                        FrameForCanvas *frameForCanvas
                                = new FrameForCanvas(rect, this);
                        objsForCanvas[mainIndex.row()].objs.insert(subRow, frameForCanvas);
                        previousRect = rect;
                        }
                        break;
                    case AbstractObj::Color:
                        {
                        ColorObj *colorObj = static_cast<ColorObj*>(subObj);
                        QRect rect = rectForControlObj(colorObj, previousRect);
                        ColorForCanvas *colorForCanvas
                                = new ColorForCanvas(rect, this);
                        objsForCanvas[mainIndex.row()].objs.insert(subRow, colorForCanvas);
                        previousRect = rect;
                        }
                        break;
                    case AbstractObj::Exp:
                        {
                        ExpObj *expObj = static_cast<ExpObj*>(subObj);
                        QRect rect = rectForControlObj(expObj, previousRect);
                        ExpForCanvas *expForCanvas
                                = new ExpForCanvas(rect, expObj->labelProperty->value(), this);
                        objsForCanvas[mainIndex.row()].objs.insert(subRow, expForCanvas);
                        previousRect = rect;
                        }
                        break;
                    case AbstractObj::ComboBox:
                        {
                        ComboBoxObj *comboBoxObj = static_cast<ComboBoxObj*>(subObj);
                        QRect rect = rectForControlObj(comboBoxObj, previousRect);
                        ComboBoxForCanvas *comboBoxForCanvas
                                = new ComboBoxForCanvas(rect,
                                                        comboBoxObj->comboBoxTypeProperty->value(),
                                                        this);
                        objsForCanvas[mainIndex.row()].objs.insert(subRow, comboBoxForCanvas);
                        previousRect = rect;
                        }
                        break;
                    case AbstractObj::File:
                        {
                        FileObj *fileObj = static_cast<FileObj*>(subObj);
                        QRect rect = rectForControlObj(fileObj, previousRect);
                        FileForCanvas *fileForCanvas
                                = new FileForCanvas(rect,
                                                    fileObj->labelProperty->value(),
                                                    fileObj->buttonWidthProperty->value(),
                                                    this);
                        objsForCanvas[mainIndex.row()].objs.insert(subRow, fileForCanvas);
                        previousRect = rect;
                        }
                        break;
                    case AbstractObj::ListBox:
                        {
                        ListBoxObj *listBoxObj = static_cast<ListBoxObj*>(subObj);
                        QRect rect = rectForControlObj(listBoxObj, previousRect);
                        ListBoxForCanvas *listBoxForCanvas
                                = new ListBoxForCanvas(rect, this);
                        objsForCanvas[mainIndex.row()].objs.insert(subRow, listBoxForCanvas);
                        previousRect = rect;
                        }
                        break;
                    default:
                        break;
                }
            }
        }
    }

    // idealWidth and idealHeight
    int nObjsOfSelectedDialog = objsForCanvas.value(rowOfSelectedDialog).objs.count();
    idealWidth = 0;
    idealHeight = 0;
    for (int i = 0; i < nObjsOfSelectedDialog; i++)
    {
        int extremeRightCoord
                = objsForCanvas.value(rowOfSelectedDialog).objs.value(i)->rect().topRight().x();
        if (extremeRightCoord > idealWidth)
            idealWidth = extremeRightCoord;
        int extremeBottomCoord
                = objsForCanvas.value(rowOfSelectedDialog).objs.value(i)->rect().bottomRight().y();
        if (extremeBottomCoord > idealHeight)
            idealHeight = extremeBottomCoord;
    }

    viewport()->update();
}

QRect Canvas::visualRect(const QModelIndex &index) const
{
    if (!index.isValid() || index.parent() == QModelIndex())
        return QRect();

    int dialogRow = index.parent().row();
    if (dialogRow == rowOfSelectedDialog)
        return viewportRectForControlObj(dialogRow, index.row());
    else return QRect();
}

QRect Canvas::viewportRectForControlObj(int dialogRow, int controlRow) const
{
    calculateRectsIfNecessary();
    QRect absoluteRect = objsForCanvas.value(dialogRow).objs.value(controlRow)->rect();
    return QRect(absoluteRect.x() - horizontalScrollBar()->value(),
                 absoluteRect.y() - verticalScrollBar()->value(),
                 absoluteRect.width(),
                 absoluteRect.height());
}

bool Canvas::isIndexHidden(const QModelIndex &index) const
{
    if (!index.isValid() || index.parent() == QModelIndex())
        return true;

    int dialogRow = index.parent().row();
    if (dialogRow != rowOfSelectedDialog)
        return true;
    else
        return false;
}

void Canvas::dataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight)
{
    hashIsDirty = true;
    QAbstractItemView::dataChanged(topLeft, bottomRight);
}

void Canvas::rowsInserted(const QModelIndex &parent, int start, int end)
{
    hashIsDirty = true;
    QAbstractItemView::rowsInserted(parent, start, end);
}

void Canvas::rowsAboutToBeRemoved(const QModelIndex &parent, int start, int end)
{
    hashIsDirty = true;
    QAbstractItemView::rowsAboutToBeRemoved(parent, start, end);
}

QModelIndex Canvas::moveCursor(CursorAction cursorAction, Qt::KeyboardModifiers modifiers)
{
    return QModelIndex();
}

int Canvas::horizontalOffset() const
{
    return horizontalScrollBar()->value();
}

int Canvas::verticalOffset() const
{
    return verticalScrollBar()->value();
}

void Canvas::scrollContentsBy(int dx, int dy)
{
    scrollDirtyRegion(dx, dy);
    viewport()->scroll(dx, dy);
}

void Canvas::setSelection(const QRect &rect, QItemSelectionModel::SelectionFlags command)
{
    Q_UNUSED(rect)
    Q_UNUSED(command)
    // Do nothing
}

void Canvas::scrollTo(const QModelIndex &index, ScrollHint hint)
{
    Q_UNUSED(index)
    Q_UNUSED(hint)
    // Do nothing
}

QModelIndex Canvas::indexAt(const QPoint &point) const
{
    if (rowOfSelectedDialog == -1)
        return QModelIndex();

    QPoint absolutePoint(point);
    absolutePoint.rx() += horizontalScrollBar()->value();
    absolutePoint.ry() += verticalScrollBar()->value();
    calculateRectsIfNecessary();
    QHash<int, AbstractObjForCanvas*> objsInSelectedDialog
            = objsForCanvas.value(rowOfSelectedDialog).objs;
    QHashIterator<int, AbstractObjForCanvas*> iter(objsInSelectedDialog);
    while (iter.hasNext())
    {
        iter.next();
        if (iter.value()->rect().contains(absolutePoint))
        {
            QModelIndex parentIndex = model()->index(rowOfSelectedDialog, 0, QModelIndex());
            return model()->index(iter.key(), 0, parentIndex);
        }
    }
    return QModelIndex();
}

void Canvas::resizeEvent(QResizeEvent *event)
{
    hashIsDirty = true;
    calculateRectsIfNecessary();
    updateGeometries();
}

void Canvas::updateGeometries()
{
    // Horizontal scroll bar
    horizontalScrollBar()->setSingleStep(2);
    horizontalScrollBar()->setPageStep(5);
    horizontalScrollBar()->setRange(0, qMax(0, idealWidth - viewport()->width()));

    // Vertical scroll bar
    verticalScrollBar()->setSingleStep(2);
    verticalScrollBar()->setPageStep(5);
    verticalScrollBar()->setRange(0, qMax(0, idealHeight - viewport()->height()));
}


QRegion Canvas::visualRegionForSelection(const QItemSelection &selection) const
{
    Q_UNUSED(selection)
    return QRegion();
}

void Canvas::mousePressEvent(QMouseEvent *event)
{
    QAbstractItemView::mousePressEvent(event);
    setCurrentIndex(indexAt(event->pos()));
}

void Canvas::onSelectionChangedInObjectListView(const QItemSelection &selected,
                                                const QItemSelection &deselected)
{
    Q_UNUSED(deselected);

    if (selected.count() == 0) return;
    if (selected.at(0).indexes().count() == 0) return;

    QModelIndex index = selected.at(0).indexes().at(0);
    AbstractObj *obj = static_cast<AbstractObj*>(index.internalPointer());
    if (obj->parentObj()->objType() == AbstractObj::Root)
    {
        if (obj->objType() == AbstractObj::Dialog)
        {
            rowOfSelectedDialog = index.row();
            selectionModel()->blockSignals(true);
            clearSelection();
            selectionModel()->blockSignals(false);
        }
        else
        {
            rowOfSelectedDialog = -1;
            selectionModel()->blockSignals(true);
            clearSelection();
            selectionModel()->blockSignals(false);
        }
    }
    else
    {
        if (obj->parentObj()->objType() == AbstractObj::Dialog)
        {
            rowOfSelectedDialog = index.parent().row();
            int rowOfSelectedControl = index.row();
            QModelIndex parentIndex = model()->index(rowOfSelectedDialog, 0);
            QModelIndex selectedIndex = model()->index(rowOfSelectedControl, 0, parentIndex);
            setCurrentIndex(selectedIndex);
        }
        else
        {
            // Should never reach this point!
            rowOfSelectedDialog = -1;
            selectionModel()->blockSignals(false);
            clearSelection();
            selectionModel()->blockSignals(false);
        }
    }

    viewport()->update();
}


void Canvas::paintEvent(QPaintEvent *event)
{
    Q_UNUSED(event)
    if (rowOfSelectedDialog == -1) return;

    QModelIndex indexOfSelectedDialog = model()->index(rowOfSelectedDialog, 0);

    calculateRectsIfNecessary();
    QPainter painter(viewport());

    // Set font
    painter.setFont(stataDialogFont);

    painter.save();
    painter.translate(-horizontalScrollBar()->value(), -verticalScrollBar()->value());
    int nObjsOfSelectedDialog = objsForCanvas.value(rowOfSelectedDialog).objs.count();
    for (int i=0; i<nObjsOfSelectedDialog; i++)
    {
        AbstractObjForCanvas *abstractObjForCanvas
                = objsForCanvas.value(rowOfSelectedDialog).objs.value(i);
        abstractObjForCanvas->draw(&painter);
        QModelIndex objIndex = model()->index(i, 0, indexOfSelectedDialog);
        QStyleOptionViewItem styleOptionViewItem = viewOptions();
        if (selectionModel()->isSelected(objIndex))
            styleOptionViewItem.state |= QStyle::State_Selected;
        if (currentIndex() == objIndex)
            styleOptionViewItem.state |= QStyle::State_HasFocus;
        itemDelegate()->paint(&painter, styleOptionViewItem, objIndex);
        if (selectionModel()->isSelected(objIndex))
            paintOutline(&painter, abstractObjForCanvas->rect());
    }
    painter.restore();
}

void Canvas::paintOutline(QPainter *painter, const QRect &rectangle)
{
    QRect rect = rectangle.adjusted(0, 0, -1, -1);
    painter->save();
    painter->setPen(QPen(QBrush(QColor(255, 0, 255)), 0.5));
    painter->drawRect(rect);
    painter->restore();
}

int Canvas::defaultDimForObj(AbstractControlObj *controlObj,
                     GeometryDataProperty::GeometryDataType geometryDataType) const
{
    switch (controlObj->objType())
    {
        case AbstractObj::Button:
            switch (geometryDataType)
            {
                case GeometryDataProperty::XSize:
                    return 80;
                    break;
                case GeometryDataProperty::YSize:
                    return 20;
                    break;
                default:
                    return 0;
                    break;
            }
            break;
        case AbstractObj::Spinner:
            switch (geometryDataType)
            {
                case GeometryDataProperty::XSize:
                    return 60;
                    break;
                case GeometryDataProperty::YSize:
                    return 20;
                    break;
                default:
                    return 0;
                    break;
            }
            break;
        case AbstractObj::CheckBox:
            switch (geometryDataType)
            {
                case GeometryDataProperty::XSize:
                    {
                    CheckBoxObj *checkBoxObj = static_cast<CheckBoxObj*>(controlObj);
                    QFontMetrics fm(stataDialogFont);
                    return 20 + fm.width(checkBoxObj->labelProperty->value());
                    }
                    break;
                case GeometryDataProperty::YSize:
                    return 20;
                    break;
                default:
                    return 0;
                    break;
            }
            break;
        case AbstractObj::Radio:
            switch (geometryDataType)
            {
                case GeometryDataProperty::XSize:
                    {
                    RadioObj *radioObj = static_cast<RadioObj*>(controlObj);
                    QFontMetrics fm(stataDialogFont);
                    return 20 + fm.width(radioObj->labelProperty->value());
                    }
                    break;
                case GeometryDataProperty::YSize:
                    return 20;
                    break;
                default:
                    return 0;
                    break;
            }
            break;
        case AbstractObj::GroupBox:
            switch (geometryDataType)
            {
                case GeometryDataProperty::XSize:
                    return 1200;
                    break;
                case GeometryDataProperty::YSize:
                    return 1200;
                    break;
                default:
                    return 0;
                    break;
            }
            break;
        case AbstractObj::Edit:
            switch (geometryDataType)
            {
                case GeometryDataProperty::XSize:
                    return 1200;
                    break;
                case GeometryDataProperty::YSize:
                    return 20;
                    break;
                default:
                    return 0;
                    break;
            }
            break;
        case AbstractObj::VarList:
            switch (geometryDataType)
            {
                case GeometryDataProperty::XSize:
                    return 210;
                    break;
                case GeometryDataProperty::YSize:
                    return 20;
                    break;
                default:
                    return 0;
                    break;
            }
            break;
        case AbstractObj::VarName:
            switch (geometryDataType)
            {
                case GeometryDataProperty::XSize:
                    return 210;
                    break;
                case GeometryDataProperty::YSize:
                    return 20;
                    break;
                default:
                    return 0;
                    break;
            }
            break;
        case AbstractObj::Text:
            switch (geometryDataType)
            {
                case GeometryDataProperty::XSize:
                    return 1200;
                    break;
                case GeometryDataProperty::YSize:
                    return 20;
                    break;
                default:
                    return 0;
                    break;
            }
            break;
        case AbstractObj::TextBox:
            switch (geometryDataType)
            {
                case GeometryDataProperty::XSize:
                    return 1200;
                    break;
                case GeometryDataProperty::YSize:
                    return 25; // I can't determine the exact value. This is a good approximation
                    break;
                default:
                    return 0;
                    break;
            }
            break;
        case AbstractObj::Frame:
            switch (geometryDataType)
            {
                case GeometryDataProperty::XSize:
                    return 1200;
                    break;
                case GeometryDataProperty::YSize:
                    return 1200;
                    break;
                default:
                    return 0;
                    break;
            }
            break;
        case AbstractObj::Color:
            switch (geometryDataType)
            {
                case GeometryDataProperty::XSize:
                    return 1200;
                    break;
                case GeometryDataProperty::YSize:
                    return 20;
                    break;
                default:
                    return 0;
                    break;
            }
            break;
        case AbstractObj::Exp:
            switch (geometryDataType)
            {
                case GeometryDataProperty::XSize:
                    return 1200;
                    break;
                case GeometryDataProperty::YSize:
                    return 20;
                    break;
                default:
                    return 0;
                    break;
            }
            break;
        case AbstractObj::ComboBox:
            switch (geometryDataType)
            {
                case GeometryDataProperty::XSize:
                    return 1200;
                    break;
                case GeometryDataProperty::YSize:
                    return 20;
                    break;
                default:
                    return 0;
                    break;
            }
            break;
        case AbstractObj::File:
            switch (geometryDataType)
            {
                case GeometryDataProperty::XSize:
                    return 1200;
                    break;
                case GeometryDataProperty::YSize:
                    return 20;
                    break;
                default:
                    return 0;
                    break;
            }
            break;
        case AbstractObj::ListBox:
            switch (geometryDataType)
            {
                case GeometryDataProperty::XSize:
                    return 1200;
                    break;
                case GeometryDataProperty::YSize:
                    return 100;
                    break;
                default:
                    return 0;
                    break;
            }
            break;
        default:
            return 0;
            break;
    }
}

int Canvas::nextGeometryValue(int previousGeometryValue,
                              AbstractControlObj *controlObj,
                              GeometryDataProperty::GeometryDataType geometryDataType) const
{
    QString geometryDataValue;
    switch (geometryDataType)
    {
        case GeometryDataProperty::X:
            geometryDataValue = controlObj->xProperty->value();
            break;
        case GeometryDataProperty::Y:
            geometryDataValue = controlObj->yProperty->value();
            break;
        case GeometryDataProperty::XSize:
            geometryDataValue = controlObj->xSizeProperty->value();
            break;
        case GeometryDataProperty::YSize:
            geometryDataValue = controlObj->ySizeProperty->value();
            break;
        default:
            break;
    }

    // Data should not be empty
    if (geometryDataValue.isEmpty())
        return 0;

    bool isNumeric;
    int numericValue = geometryDataValue.toInt(&isNumeric);

    if (isNumeric && geometryDataValue.at(0) != '+' && geometryDataValue.at(0) != '-') // No extra code
        return numericValue;
    else if (geometryDataValue == ".") // Default value
        return defaultDimForObj(controlObj, geometryDataType);
    else if (geometryDataValue == "@") // Previous value
        return previousGeometryValue;
    else if (geometryDataValue.at(0) == '+') // Positive offset
        return previousGeometryValue + numericValue;
    else if (geometryDataValue.at(0) == '-') // Negative offset
        return previousGeometryValue + numericValue;
    else // Name
        return 0;
}

QRect Canvas::rectForControlObj(AbstractControlObj *controlObj, QRect previousRect) const
{
    QRect rect;

    rect.setX(nextGeometryValue(previousRect.x(), controlObj, GeometryDataProperty::X));
    rect.setY(nextGeometryValue(previousRect.y(), controlObj, GeometryDataProperty::Y));
    rect.setWidth(nextGeometryValue(previousRect.width(), controlObj, GeometryDataProperty::XSize));
    rect.setHeight(nextGeometryValue(previousRect.height(), controlObj, GeometryDataProperty::YSize));

    return rect;
}

bool Canvas::isControlObjType(AbstractObj::ObjType objType)
{
    switch (objType)
    {
        case AbstractObj::CheckBox:
        case AbstractObj::Radio:
        case AbstractObj::Spinner:
        case AbstractObj::Edit:
        case AbstractObj::VarList:
        case AbstractObj::VarName:
        case AbstractObj::File:
        case AbstractObj::ListBox:
        case AbstractObj::ComboBox:
        case AbstractObj::Button:
        case AbstractObj::Text:
        case AbstractObj::TextBox:
        case AbstractObj::GroupBox:
        case AbstractObj::Frame:
        case AbstractObj::Color:
        case AbstractObj::Exp:
            return true;
            break;
        default:
            return false;
            break;
    }
}

void Canvas::dropEvent(QDropEvent *event)
{
    const ObjTypeMimeData *objTypeMimeData =
            qobject_cast<const ObjTypeMimeData*>(event->mimeData());
    AbstractObj::ObjType objType = objTypeMimeData->objType();
    if (!isControlObjType(objType))
        return;
    if (rowOfSelectedDialog == -1)
        return;

    QModelIndex dialogIndex = model()->index(rowOfSelectedDialog, 0);
    Model *innerModel = static_cast<Model*>(this->model());

    innerModel->appendNewObj(objType, dialogIndex, event->pos().x(), event->pos().y());
    int childCount = model()->rowCount(dialogIndex);
    QModelIndex newIndex = model()->index(childCount-1, 0, dialogIndex);
    setCurrentIndex(newIndex);

    event->acceptProposedAction();
}

void Canvas::resetRowOfSelectedDialog()
{
    rowOfSelectedDialog = -1;
    selectionModel()->blockSignals(true);
    clearSelection();
    selectionModel()->blockSignals(false);
}
