#include "document.h"
#include <QFile>
#include <QDomDocument>
#include <QTextStream>

// Objects
#include "versionobj.h"
#include "includeobj.h"
#include "defineobj.h"
#include "positionobj.h"
#include "listobj.h"
#include "dialogobj.h"
#include "checkboxobj.h"
#include "radioobj.h"
#include "spinnerobj.h"
#include "editobj.h"
#include "varlistobj.h"
#include "varnameobj.h"
#include "fileobj.h"
#include "listboxobj.h"
#include "comboboxobj.h"
#include "buttonobj.h"
#include "textobj.h"
#include "textboxobj.h"
#include "groupboxobj.h"
#include "frameobj.h"
#include "colorobj.h"
#include "expobj.h"
#include "okobj.h"
#include "submitobj.h"
#include "copyobj.h"
#include "cancelobj.h"
#include "helpobj.h"
#include "resetobj.h"
#include "scriptobj.h"
#include "programobj.h"

Document::Document()
{
    m_rootObj = new RootObj;
}

void Document::appendObj(AbstractObj* obj)
{
    m_rootObj->appendObj(obj);
}

void Document::insertObj(AbstractObj *obj, int pos)
{
    m_rootObj->insertObj(obj, pos);
}

void Document::swapObjects(int parentRow, int row1, int row2)
{
    if (parentRow == -1)
        m_rootObj->swapObjects(row1, row2);
    else
        m_rootObj->objAt(parentRow)->swapObjects(row1, row2);
}

void Document::moveObjUp(int parentRow, int row)
{
    if (parentRow == -1)
        m_rootObj->swapObjects(row, row - 1);
    else
        m_rootObj->objAt(parentRow)->swapObjects(row, row - 1);
}

void Document::moveObjDown(int parentRow, int row)
{
    if (parentRow == -1)
        m_rootObj->swapObjects(row, row + 1);
    else
        m_rootObj->objAt(parentRow)->swapObjects(row, row + 1);
}

void Document::deleteObjAt(int pos)
{
    if (pos>=0 && pos<m_rootObj->objCount())
    {
        m_rootObj->deleteObjAt(pos);
    }
}

AbstractObj *Document::objAt(int pos)
{
    return m_rootObj->objAt(pos);
}

int Document::objCount() const
{
    return m_rootObj->objCount();
}

QString Document::commandString() const
{
    QString string;
    for (int i=0; i<m_rootObj->objCount(); i++)
        string += m_rootObj->objAt(i)->commandString() + "\n";
    return string;
}

bool Document::writeVisuaFile(const QString &fileName) const
{
    QFile file(fileName);
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
    {
        file.close();
        return false;
    }

    QDomDocument xmlDocument;

    QDomElement rootDomElement = xmlDocument.createElement("visua");
    rootDomElement.setAttribute("version", "0.1");
    for (int i=0; i<m_rootObj->objCount(); i++)
        rootDomElement.appendChild(m_rootObj->objAt(i)->domElement(xmlDocument));

    xmlDocument.appendChild(rootDomElement);
    QTextStream textStream(&file);
    xmlDocument.save(textStream, 4);
    file.close();
    return true;
}

bool Document::loadVisuaFile(const QString &fileName)
{
    QFile file(fileName);
    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        file.close();
        return false;
    }
    QDomDocument xmlDocument;
    if (!xmlDocument.setContent(&file))
    {
        file.close();
        return false;
    }
    QDomElement root = xmlDocument.documentElement();
    if (root.tagName() != "visua")
    {
        file.close();
        return false;
    }
    if (root.attribute("version") != "0.1")
    {
        file.close();
        return false;
    }

    // Clear document
    clear();

    QDomNode node = root.firstChild();
    while (!node.isNull())
    {
        if (node.toElement().tagName() == "object")
        {
            QDomElement domElement = node.toElement();
            QString objectType = domElement.attributes().namedItem("type").nodeValue();
            AbstractObj *obj = 0;
            if (objectType == "version") obj = new VersionObj;
            if (objectType == "include") obj = new IncludeObj;
            if (objectType == "define") obj = new DefineObj;
            if (objectType == "position") obj = new PositionObj;
            if (objectType == "list") obj = new ListObj;
            if (objectType == "dialog") obj = new DialogObj;
            if (objectType == "checkbox") obj = new CheckBoxObj;
            if (objectType == "radio") obj = new RadioObj;
            if (objectType == "spinner") obj = new SpinnerObj;
            if (objectType == "edit") obj = new EditObj;
            if (objectType == "varlist") obj = new VarListObj;
            if (objectType == "file") obj = new FileObj;
            if (objectType == "listbox") obj = new ListBoxObj;
            if (objectType == "combobox") obj = new ComboBoxObj;
            if (objectType == "button") obj = new ButtonObj;
            if (objectType == "text") obj = new TextObj;
            if (objectType == "textbox") obj = new TextBoxObj;
            if (objectType == "groupbox") obj = new GroupBoxObj;
            if (objectType == "frame") obj = new FrameObj;
            if (objectType == "color") obj = new ColorObj;
            if (objectType == "exp") obj = new ExpObj;
            if (objectType == "ok") obj = new OkObj;
            if (objectType == "submit") obj = new SubmitObj;
            if (objectType == "copy") obj = new CopyObj;
            if (objectType == "cancel") obj = new CancelObj;
            if (objectType == "help") obj = new HelpObj;
            if (objectType == "reset") obj = new ResetObj;
            if (objectType == "script") obj = new ScriptObj;
            if (objectType == "program") obj = new ProgramObj;
            if (obj)
            {
                obj->setFromObjDomElement(domElement);
                m_rootObj->appendObj(obj);
            }
        }
        node = node.nextSibling();
    }
    file.close();
    return true;
}

void Document::clear()
{
    delete m_rootObj;
    m_rootObj = new RootObj;
}

bool Document::writeStataDlgCodeOnFile(const QString &fileName) const
{
    QFile file(fileName);
    if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
        return false;
    QTextStream dataStream(&file);
    dataStream << commandString();
    return true;
}

bool Document::hasAtLeastOneChildControlObjBeforeRow(int row) const
{
    for (int i=0; i<row; i++)
    {
        if (m_rootObj->objAt(i)->isControlObj())
            return true;
    }
    return false;
}

QList<AbstractObj*> Document::objListFromVisuaCode(const QString &visuaCode)
{
    QList<AbstractObj*> objList;

    QDomDocument xmlDocument;
    if (!xmlDocument.setContent(visuaCode))
    {
        return objList;
    }
    QDomElement root = xmlDocument.documentElement();
    if (root.tagName() != "visua")
    {
        return objList;
    }

    QDomNode node = root.firstChild();
    while (!node.isNull())
    {
        if (node.toElement().tagName() == "object")
        {
            QDomElement domElement = node.toElement();
            QString objectType = domElement.attributes().namedItem("type").nodeValue();
            AbstractObj *obj = 0;
            if (objectType == "version") obj = new VersionObj;
            if (objectType == "include") obj = new IncludeObj;
            if (objectType == "define") obj = new DefineObj;
            if (objectType == "position") obj = new PositionObj;
            if (objectType == "list") obj = new ListObj;
            if (objectType == "dialog") obj = new DialogObj;
            if (objectType == "checkbox") obj = new CheckBoxObj;
            if (objectType == "radio") obj = new RadioObj;
            if (objectType == "spinner") obj = new SpinnerObj;
            if (objectType == "edit") obj = new EditObj;
            if (objectType == "varlist") obj = new VarListObj;
            if (objectType == "file") obj = new FileObj;
            if (objectType == "listbox") obj = new ListBoxObj;
            if (objectType == "combobox") obj = new ComboBoxObj;
            if (objectType == "button") obj = new ButtonObj;
            if (objectType == "text") obj = new TextObj;
            if (objectType == "textbox") obj = new TextBoxObj;
            if (objectType == "groupbox") obj = new GroupBoxObj;
            if (objectType == "frame") obj = new FrameObj;
            if (objectType == "color") obj = new ColorObj;
            if (objectType == "exp") obj = new ExpObj;
            if (objectType == "ok") obj = new OkObj;
            if (objectType == "submit") obj = new SubmitObj;
            if (objectType == "copy") obj = new CopyObj;
            if (objectType == "cancel") obj = new CancelObj;
            if (objectType == "help") obj = new HelpObj;
            if (objectType == "reset") obj = new ResetObj;
            if (objectType == "script") obj = new ScriptObj;
            if (objectType == "program") obj = new ProgramObj;
            if (obj)
            {
                obj->setFromObjDomElement(domElement);
                objList.append(obj);
            }
        }
        node = node.nextSibling();
    }

    return objList;
}


