#include "iactionsetter.h"

#include <QHBoxLayout>
#include <QMenu>
#include <QCompleter>

IActionSetter::IActionSetter(AbstractObj::ObjType objType, IActionCompleter *completer,
                             const QAbstractItemModel *sourceModel, QWidget *parent)
    : AbstractSetter(parent)
{
    // Layout
    QHBoxLayout *layout = new QHBoxLayout;
    layout->setMargin(0);
    layout->setContentsMargins(0, 0, 0, 0);
    layout->setSpacing(0);
    setLayout(layout);

    // Create member function actions for controls
    createMemberFunctionActionsForControls();

    // Main menu
    mainMenu = new QMenu(this);

    // Action menu
    createActionMenu(sourceModel);

    // Create member functions menus
    createMemberFunctionMenus();

    // Member function for this control menu
    if (objType != AbstractObj::Root)
        createMemberFunctionForThisControlMenu(objType);

    // Other action menu
    createOtherActionMenu();

    // Do nothing action
    QAction *doNothingAction = new QAction("Do nothing", this);
    connect(doNothingAction, SIGNAL(triggered()), this, SLOT(onDoNothingActionTriggered()));
    mainMenu->addAction(doNothingAction);

    // Clear all action
    QAction *clearAllAction = new QAction("Clear all", this);
    clearAllAction->setToolTip(tr("ciao"));

    connect(clearAllAction, SIGNAL(triggered()), this, SLOT(onClearAllActionTriggered()));
    mainMenu->addAction(clearAllAction);

    // Line edit
    lineEdit = new QLineEdit;
    lineEdit->setCompleter(completer);
    lineEdit->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    lineEdit->setFocus();
    lineEdit->setCursorPosition(1);
    layout->addWidget(lineEdit);

    // Button
    button = new QPushButton;
    button->setMenu(mainMenu);
    button->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
    button->setMinimumWidth(20);
    button->setMaximumWidth(20);
    layout->addWidget(button);
}

void IActionSetter::createOtherActionMenu()
{
    QMenu *otherActionMenu = new QMenu("Other action", this);
    mainMenu->addMenu(otherActionMenu);

    // View topic
    QAction *viewTopicAction = new QAction("view", this);
    viewTopicAction->setData("view");
    otherActionMenu->addAction(viewTopicAction);

    // Script
    QAction *scriptAction = new QAction("script", this);
    scriptAction->setData("script");
    otherActionMenu->addAction(scriptAction);

    for (int i=0; i<otherActionMenu->actions().count(); i++)
        connect(otherActionMenu->actions().at(i), SIGNAL(triggered()),
                this, SLOT(onIActionChosen_SetText()));
}

void IActionSetter::onDoNothingActionTriggered()
{
    lineEdit->setText(".");
}

void IActionSetter::setIActionData(const QString &data)
{
    lineEdit->setText(data);
}

QString IActionSetter::iActionData() const
{
    return lineEdit->text();
}

void IActionSetter::createMemberFunctionForThisControlMenu(AbstractObj::ObjType objType)
{
    QMenu *menu = new QMenu("Member functions for this control", this);
    mainMenu->addMenu(menu);

    QList<QAction*> actionsForThisControl;
    QList<QAction*> actionsToCopy = getActionsForControl(objType);
    for (int k=0; k<actionsToCopy.count(); k++)
    {
        QAction *action = new QAction(actionsToCopy.at(k)->text(), this);
        actionsForThisControl.append(action);
        action->setData(actionsToCopy.at(k)->data());
        connect(action, SIGNAL(triggered()), this, SLOT(onIActionChosen_SetText()));
    }
    menu->addActions(actionsForThisControl);
}


void IActionSetter::onIActionChosen_SetText()
{
    QAction *action = qobject_cast<QAction *>(sender());
    QString text = action->data().toString();
    int firstSharpPosition = text.indexOf("@");
    int secondSharpPosition = -1;
    if (firstSharpPosition != -1)
    {
        text.replace(firstSharpPosition, 1, "");
        secondSharpPosition = text.indexOf("@");
        text.replace(secondSharpPosition, 1, "");
    }

    lineEdit->setText(text);
    if (firstSharpPosition != -1)
    {
        lineEdit->setSelection(firstSharpPosition, secondSharpPosition - firstSharpPosition);
    }
    lineEdit->setFocus();
}

void IActionSetter::onIActionChosen_InsertText()
{
    QAction *action = qobject_cast<QAction *>(sender());
    QString text = action->data().toString();
    int firstSharpPosition = text.indexOf("@");
    int secondSharpPosition = -1;
    if (firstSharpPosition != -1)
    {
        text.replace(firstSharpPosition, 1, "");
        secondSharpPosition = text.indexOf("@");
        text.replace(secondSharpPosition, 1, "");
    }

    int cursorPosition = lineEdit->cursorPosition();
    lineEdit->insert(text);
    if (firstSharpPosition != -1)
    {
        lineEdit->setSelection(firstSharpPosition + cursorPosition,
                               secondSharpPosition - firstSharpPosition);
    }
    lineEdit->setFocus();
}

void IActionSetter::createMemberFunctionActionsForControls()
{
    // checkbox
    checkboxIActions.append(memberFunctionAction("hide", "hide"));
    checkboxIActions.append(memberFunctionAction("show", "show"));
    checkboxIActions.append(memberFunctionAction("disable", "disable"));
    checkboxIActions.append(memberFunctionAction("enable", "enable"));
    checkboxIActions.append(memberFunctionAction("setposition", "setposition @x y xsize ysize@"));
    checkboxIActions.append(memberFunctionAction("setlabel", "setlabel \"@string@\""));
    checkboxIActions.append(memberFunctionAction("setoff", "setoff"));
    checkboxIActions.append(memberFunctionAction("seton", "seton"));
    checkboxIActions.append(memberFunctionAction("setoption", "setoption @optionname@"));
    checkboxIActions.append(memberFunctionAction("setdefault", "setdefault @value@"));
    checkboxIActions.append(memberFunctionAction("settooltip", "settooltip \"@string@\""));

    // radio
    radioIActions.append(memberFunctionAction("hide", "hide"));
    radioIActions.append(memberFunctionAction("show", "show"));
    radioIActions.append(memberFunctionAction("disable", "disable"));
    radioIActions.append(memberFunctionAction("enable", "enable"));
    radioIActions.append(memberFunctionAction("setposition", "setposition @x y xsize ysize@"));
    radioIActions.append(memberFunctionAction("setlabel", "setlabel \"@string@\""));
    radioIActions.append(memberFunctionAction("seton", "seton"));
    radioIActions.append(memberFunctionAction("setoption", "setoption @optionname@"));
    radioIActions.append(memberFunctionAction("setdefault", "setdefault @value@"));
    radioIActions.append(memberFunctionAction("settooltip", "settooltip \"@string@\""));

    // spinner
    spinnerIActions.append(memberFunctionAction("hide", "hide"));
    spinnerIActions.append(memberFunctionAction("show", "show"));
    spinnerIActions.append(memberFunctionAction("disable", "disable"));
    spinnerIActions.append(memberFunctionAction("enable", "enable"));
    spinnerIActions.append(memberFunctionAction("setposition", "setposition @x y xsize ysize@"));
    spinnerIActions.append(memberFunctionAction("setvalue", "setvalue @value@"));
    spinnerIActions.append(memberFunctionAction("setrange", "setrange @min# max#@"));
    spinnerIActions.append(memberFunctionAction("setoption", "setoption @optionname@"));
    spinnerIActions.append(memberFunctionAction("setdefault", "setdefault @#@"));
    spinnerIActions.append(memberFunctionAction("settooltip", "settooltip \"@string@\""));

    // edit
    editIActions.append(memberFunctionAction("hide", "hide"));
    editIActions.append(memberFunctionAction("show", "show"));
    editIActions.append(memberFunctionAction("disable", "disable"));
    editIActions.append(memberFunctionAction("enable", "enable"));
    editIActions.append(memberFunctionAction("setposition", "setposition @x y xsize ysize@"));
    editIActions.append(memberFunctionAction("setlabel", "setlabel \"@string@\""));
    editIActions.append(memberFunctionAction("setvalue", "setvalue @strvalue@"));
    editIActions.append(memberFunctionAction("append", "append \"@string@\""));
    editIActions.append(memberFunctionAction("prepend", "prepend \"@string@\""));
    editIActions.append(memberFunctionAction("insert", "insert \"@string@\""));
    editIActions.append(memberFunctionAction("smartinsert", "smartinsert \"@string@\""));
    editIActions.append(memberFunctionAction("setfocus", "setfocus"));
    editIActions.append(memberFunctionAction("setoption", "setoption @optionname@"));
    editIActions.append(memberFunctionAction("setdefault", "setdefault \"@string@\""));
    editIActions.append(memberFunctionAction("settooltip", "settooltip \"@string@\""));

    // varlist
    varlistIActions.append(memberFunctionAction("hide", "hide"));
    varlistIActions.append(memberFunctionAction("show", "show"));
    varlistIActions.append(memberFunctionAction("disable", "disable"));
    varlistIActions.append(memberFunctionAction("enable", "enable"));
    varlistIActions.append(memberFunctionAction("setposition", "setposition @x y xsize ysize@"));
    varlistIActions.append(memberFunctionAction("setlabel", "setlabel \"@string@\""));
    varlistIActions.append(memberFunctionAction("setvalue", "setvalue @strvalue@"));
    varlistIActions.append(memberFunctionAction("append", "append \"@string@\""));
    varlistIActions.append(memberFunctionAction("prepend", "prepend \"@string@\""));
    varlistIActions.append(memberFunctionAction("insert", "insert \"@string@\""));
    varlistIActions.append(memberFunctionAction("smartinsert", "smartinsert \"@string@\""));
    varlistIActions.append(memberFunctionAction("setfocus", "setfocus"));
    varlistIActions.append(memberFunctionAction("setoption", "setoption @optionname@"));
    varlistIActions.append(memberFunctionAction("setdefault", "setdefault \"@string@\""));
    varlistIActions.append(memberFunctionAction("settooltip", "settooltip \"@string@\""));

    // varname
    varnameIActions.append(memberFunctionAction("hide", "hide"));
    varnameIActions.append(memberFunctionAction("show", "show"));
    varnameIActions.append(memberFunctionAction("disable", "disable"));
    varnameIActions.append(memberFunctionAction("enable", "enable"));
    varnameIActions.append(memberFunctionAction("setposition", "setposition @x y xsize ysize@"));
    varnameIActions.append(memberFunctionAction("setlabel", "setlabel \"@string@\""));
    varnameIActions.append(memberFunctionAction("setvalue", "setvalue @strvalue@"));
    varnameIActions.append(memberFunctionAction("append", "append \"@string@\""));
    varnameIActions.append(memberFunctionAction("prepend", "prepend \"@string@\""));
    varnameIActions.append(memberFunctionAction("insert", "insert \"@string@\""));
    varnameIActions.append(memberFunctionAction("smartinsert", "smartinsert \"@string@\""));
    varnameIActions.append(memberFunctionAction("setfocus", "setfocus"));
    varnameIActions.append(memberFunctionAction("setoption", "setoption @optionname@"));
    varnameIActions.append(memberFunctionAction("setdefault", "setdefault \"@string@\""));
    varnameIActions.append(memberFunctionAction("settooltip", "settooltip \"@string@\""));

    // file
    fileIActions.append(memberFunctionAction("hide", "hide"));
    fileIActions.append(memberFunctionAction("show", "show"));
    fileIActions.append(memberFunctionAction("disable", "disable"));
    fileIActions.append(memberFunctionAction("enable", "enable"));
    fileIActions.append(memberFunctionAction("setposition", "setposition @x y xsize ysize@"));
    fileIActions.append(memberFunctionAction("setlabel", "setlabel \"@string@\""));
    fileIActions.append(memberFunctionAction("setvalue", "setvalue @strvalue@"));
    fileIActions.append(memberFunctionAction("append", "append \"@string@\""));
    fileIActions.append(memberFunctionAction("prepend", "prepend \"@string@\""));
    fileIActions.append(memberFunctionAction("insert", "insert \"@string@\""));
    fileIActions.append(memberFunctionAction("smartinsert", "smartinsert \"@string@\""));
    fileIActions.append(memberFunctionAction("setoption", "setoption @optionname@"));
    fileIActions.append(memberFunctionAction("setdefault", "setdefault \"@string@\""));
    fileIActions.append(memberFunctionAction("settooltip", "settooltip \"@string@\""));

    // listbox
    listboxIActions.append(memberFunctionAction("hide", "hide"));
    listboxIActions.append(memberFunctionAction("show", "show"));
    listboxIActions.append(memberFunctionAction("disable", "disable"));
    listboxIActions.append(memberFunctionAction("enable", "enable"));
    listboxIActions.append(memberFunctionAction("setposition", "setposition @x y xsize ysize@"));
    listboxIActions.append(memberFunctionAction("setlabel", "setlabel \"@string@\""));
    listboxIActions.append(memberFunctionAction("setvalue", "setvalue @strvalue@"));
    listboxIActions.append(memberFunctionAction("setfocus", "setfocus"));
    listboxIActions.append(memberFunctionAction("setoption", "setoption @optionname@"));
    listboxIActions.append(memberFunctionAction("setdefault", "setdefault \"@string@\""));
    listboxIActions.append(memberFunctionAction("repopulate", "repopulate"));
    listboxIActions.append(memberFunctionAction("forceselchange", "forceselchange"));
    listboxIActions.append(memberFunctionAction("settooltip", "settooltip \"@string@\""));

    // combobox
    comboboxIActions.append(memberFunctionAction("hide", "hide"));
    comboboxIActions.append(memberFunctionAction("show", "show"));
    comboboxIActions.append(memberFunctionAction("disable", "disable"));
    comboboxIActions.append(memberFunctionAction("enable", "enable"));
    comboboxIActions.append(memberFunctionAction("setposition", "setposition @x y xsize ysize@"));
    comboboxIActions.append(memberFunctionAction("setlabel", "setlabel \"@string@\""));
    comboboxIActions.append(memberFunctionAction("setvalue", "setvalue @strvalue@"));
    comboboxIActions.append(memberFunctionAction("setfocus", "setfocus"));
    comboboxIActions.append(memberFunctionAction("setoption", "setoption @optionname@"));
    comboboxIActions.append(memberFunctionAction("setdefault", "setdefault \"@string@\""));
    comboboxIActions.append(memberFunctionAction("repopulate", "repopulate"));
    comboboxIActions.append(memberFunctionAction("forceselchange", "forceselchange"));
    comboboxIActions.append(memberFunctionAction("settooltip", "settooltip \"@string@\""));
    comboboxIActions.append(memberFunctionAction("append", "append \"@string@\""));
    comboboxIActions.append(memberFunctionAction("prepend", "prepend \"@string@\""));
    comboboxIActions.append(memberFunctionAction("insert", "insert \"@string@\""));
    comboboxIActions.append(memberFunctionAction("smartinsert", "smartinsert \"@string@\""));

    // button
    buttonIActions.append(memberFunctionAction("hide", "hide"));
    buttonIActions.append(memberFunctionAction("show", "show"));
    buttonIActions.append(memberFunctionAction("disable", "disable"));
    buttonIActions.append(memberFunctionAction("enable", "enable"));
    buttonIActions.append(memberFunctionAction("setposition", "setposition @x y xsize ysize@"));
    buttonIActions.append(memberFunctionAction("setlabel", "setlabel \"@string@\""));
    buttonIActions.append(memberFunctionAction("setfocus", "setfocus"));
    buttonIActions.append(memberFunctionAction("settooltip", "settooltip \"@string@\""));

    // text
    textIActions.append(memberFunctionAction("hide", "hide"));
    textIActions.append(memberFunctionAction("show", "show"));
    textIActions.append(memberFunctionAction("disable", "disable"));
    textIActions.append(memberFunctionAction("enable", "enable"));
    textIActions.append(memberFunctionAction("setposition", "setposition @x y xsize ysize@"));
    textIActions.append(memberFunctionAction("setlabel", "setlabel \"@string@\""));

    // textbox
    textboxIActions.append(memberFunctionAction("hide", "hide"));
    textboxIActions.append(memberFunctionAction("show", "show"));
    textboxIActions.append(memberFunctionAction("disable", "disable"));
    textboxIActions.append(memberFunctionAction("enable", "enable"));
    textboxIActions.append(memberFunctionAction("setposition", "setposition @x y xsize ysize@"));
    textboxIActions.append(memberFunctionAction("setlabel", "setlabel \"@string@\""));

    // groupbox
    groupboxIActions.append(memberFunctionAction("hide", "hide"));
    groupboxIActions.append(memberFunctionAction("show", "show"));
    groupboxIActions.append(memberFunctionAction("disable", "disable"));
    groupboxIActions.append(memberFunctionAction("enable", "enable"));
    groupboxIActions.append(memberFunctionAction("setposition", "setposition @x y xsize ysize@"));
    groupboxIActions.append(memberFunctionAction("setlabel", "setlabel \"@string@\""));

    // frame
    frameIActions.append(memberFunctionAction("hide", "hide"));
    frameIActions.append(memberFunctionAction("show", "show"));
    frameIActions.append(memberFunctionAction("disable", "disable"));
    frameIActions.append(memberFunctionAction("enable", "enable"));
    frameIActions.append(memberFunctionAction("setposition", "setposition @x y xsize ysize@"));

    // color
    colorIActions.append(memberFunctionAction("hide", "hide"));
    colorIActions.append(memberFunctionAction("show", "show"));
    colorIActions.append(memberFunctionAction("disable", "disable"));
    colorIActions.append(memberFunctionAction("enable", "enable"));
    colorIActions.append(memberFunctionAction("setposition", "setposition @x y xsize ysize@"));
    colorIActions.append(memberFunctionAction("setvalue", "setvalue @rgbvalue@"));
    colorIActions.append(memberFunctionAction("setoption", "setoption @optionname@"));
    colorIActions.append(memberFunctionAction("setdefault", "setdefault @rgbvalue@"));
    colorIActions.append(memberFunctionAction("settooltip", "settooltip \"@string@\""));

    // exp
    expIActions.append(memberFunctionAction("hide", "hide"));
    expIActions.append(memberFunctionAction("show", "show"));
    expIActions.append(memberFunctionAction("disable", "disable"));
    expIActions.append(memberFunctionAction("enable", "enable"));
    expIActions.append(memberFunctionAction("setposition", "setposition @x y xsize ysize@"));
    expIActions.append(memberFunctionAction("setlabel", "setlabel \"@string@\""));
    expIActions.append(memberFunctionAction("setvalue", "setvalue @strvalue@"));
    expIActions.append(memberFunctionAction("append", "append \"@string@\""));
    expIActions.append(memberFunctionAction("prepend", "prepend \"@string@\""));
    expIActions.append(memberFunctionAction("insert", "insert \"@string@\""));
    expIActions.append(memberFunctionAction("smartinsert", "smartinsert \"@string@\""));
    expIActions.append(memberFunctionAction("setoption", "setoption @optionname@"));
    expIActions.append(memberFunctionAction("setdefault", "setdefault \"@string@\""));
    expIActions.append(memberFunctionAction("settooltip", "settooltip \"@string@\""));
}

QAction *IActionSetter::memberFunctionAction(const QString &displayText, const QString &actionData)
{
    QAction *action = new QAction(displayText, this);
    action->setData(actionData);
    connect(action, SIGNAL(triggered()), this, SLOT(onIActionChosen_InsertText()));
    return action;
}

void IActionSetter::createMemberFunctionMenus()
{
    QMenu *memberFunctionMenu = new QMenu("Member function", this);
    mainMenu->addMenu(memberFunctionMenu);

    // checkbox
    QMenu *checkboxMemberFunctionsMenu = new QMenu("(Checkbox)", this);
    memberFunctionMenu->addMenu(checkboxMemberFunctionsMenu);
    checkboxMemberFunctionsMenu->addActions(checkboxIActions);

    // radio
    QMenu *radioMemberFunctionsMenu = new QMenu("(Radio)", memberFunctionMenu);
    memberFunctionMenu->addMenu(radioMemberFunctionsMenu);
    radioMemberFunctionsMenu->addActions(radioIActions);

    // spinner
    QMenu *spinnerMemberFunctionsMenu = new QMenu("(Spinner)", memberFunctionMenu);
    memberFunctionMenu->addMenu(spinnerMemberFunctionsMenu);
    spinnerMemberFunctionsMenu->addActions(spinnerIActions);

    // edit
    QMenu *editMemberFunctionsMenu = new QMenu("(Edit)", memberFunctionMenu);
    memberFunctionMenu->addMenu(editMemberFunctionsMenu);
    editMemberFunctionsMenu->addActions(editIActions);

    // varlist
    QMenu *varlistMemberFunctionsMenu = new QMenu("(Varlist)", memberFunctionMenu);
    memberFunctionMenu->addMenu(varlistMemberFunctionsMenu);
    varlistMemberFunctionsMenu->addActions(varlistIActions);

    // varname
    QMenu *varnameMemberFunctionsMenu = new QMenu("(Varname)", memberFunctionMenu);
    memberFunctionMenu->addMenu(varnameMemberFunctionsMenu);
    varnameMemberFunctionsMenu->addActions(varnameIActions);

    // file
    QMenu *fileMemberFunctionsMenu = new QMenu("(File)", memberFunctionMenu);
    memberFunctionMenu->addMenu(fileMemberFunctionsMenu);
    fileMemberFunctionsMenu->addActions(fileIActions);

    // listbox
    QMenu *listboxMemberFunctionsMenu = new QMenu("(List box)", memberFunctionMenu);
    memberFunctionMenu->addMenu(listboxMemberFunctionsMenu);
    listboxMemberFunctionsMenu->addActions(listboxIActions);

    // combobox
    QMenu *comboboxMemberFunctionsMenu = new QMenu("(Combo box)", memberFunctionMenu);
    memberFunctionMenu->addMenu(comboboxMemberFunctionsMenu);
    comboboxMemberFunctionsMenu->addActions(comboboxIActions);

    // button
    QMenu *buttonMemberFunctionsMenu = new QMenu("(Button)", memberFunctionMenu);
    memberFunctionMenu->addMenu(buttonMemberFunctionsMenu);
    buttonMemberFunctionsMenu->addActions(buttonIActions);

    // text
    QMenu *textMemberFunctionsMenu = new QMenu("(Text)", memberFunctionMenu);
    memberFunctionMenu->addMenu(textMemberFunctionsMenu);
    textMemberFunctionsMenu->addActions(textIActions);

    // textbox
    QMenu *textboxMemberFunctionsMenu = new QMenu("(Text box)", memberFunctionMenu);
    memberFunctionMenu->addMenu(textboxMemberFunctionsMenu);
    textboxMemberFunctionsMenu->addActions(textboxIActions);

    // groupbox
    QMenu *groupboxMemberFunctionsMenu = new QMenu("(Group box)", memberFunctionMenu);
    memberFunctionMenu->addMenu(groupboxMemberFunctionsMenu);
    groupboxMemberFunctionsMenu->addActions(groupboxIActions);

    // frame
    QMenu *frameMemberFunctionsMenu = new QMenu("(Frame)", memberFunctionMenu);
    memberFunctionMenu->addMenu(frameMemberFunctionsMenu);
    frameMemberFunctionsMenu->addActions(frameIActions);

    // color
    QMenu *colorMemberFunctionsMenu = new QMenu("(Color)", memberFunctionMenu);
    memberFunctionMenu->addMenu(colorMemberFunctionsMenu);
    colorMemberFunctionsMenu->addActions(colorIActions);

    // exp
    QMenu *expMemberFunctionsMenu = new QMenu("(Exp)", memberFunctionMenu);
    memberFunctionMenu->addMenu(expMemberFunctionsMenu);
    expMemberFunctionsMenu->addActions(expIActions);
}

void IActionSetter::onClearAllActionTriggered()
{
    lineEdit->clear();
}

void IActionSetter::createActionMenu(const QAbstractItemModel *sourceModel)
{
    QMenu *actionMenu = new QMenu("Action", this);
    mainMenu->addMenu(actionMenu);

    for (int i=0; i<sourceModel->rowCount(); i++)
    {
        AbstractObj *obj = static_cast<AbstractObj*>(sourceModel->index(i, 0).internalPointer());
        if (obj->objType() == AbstractObj::Dialog)
        {
            QString dialogName = sourceModel->index(i, 1).data().toString();
            QMenu *dialogSubMenu = new QMenu(dialogName, this);
            actionMenu->addMenu(dialogSubMenu);
            int nObjInDialog = sourceModel->rowCount(sourceModel->index(i, 0));
            QModelIndex dialogParentIndex = sourceModel->index(i, 0);
            for (int j=0; j<nObjInDialog; j++)
            {
                AbstractObj *controlObj
                    = static_cast<AbstractObj*>(sourceModel->index(j, 0,
                      dialogParentIndex).internalPointer());
                AbstractObj::ObjType controlObjType = controlObj->objType();
                switch (controlObjType)
                {
                    case AbstractObj::CheckBox:
                    case AbstractObj::Radio:
                    case AbstractObj::Spinner:
                    case AbstractObj::Edit:
                    case AbstractObj::VarList:
                    case AbstractObj::VarName:
                    case AbstractObj::File:
                    case AbstractObj::ListBox:
                    case AbstractObj::ComboBox:
                    case AbstractObj::Button:
                    case AbstractObj::Text:
                    case AbstractObj::TextBox:
                    case AbstractObj::GroupBox:
                    case AbstractObj::Frame:
                    case AbstractObj::Color:
                    case AbstractObj::Exp:
                        {
                        QString controlName =
                                sourceModel->index(j, 1, dialogParentIndex).data().toString();
                        QMenu *controlSubMenu =
                                new QMenu(controlName +
                                          " (" + controlObj->commandNameForDisplayRoleData() + ")", this);
                        dialogSubMenu->addMenu(controlSubMenu);
                        QList<QAction*> actionsForControl;
                        QList<QAction*> actionsToCopy = getActionsForControl(controlObjType);
                        for (int k=0; k<actionsToCopy.count(); k++)
                        {
                            QAction *action = new QAction(actionsToCopy.at(k)->text(), this);
                            actionsForControl.append(action);
                            QString originalData = actionsToCopy.at(k)->data().toString();
                            QString newData = dialogName + "." + controlName + "." + originalData;
                            action->setData(newData);
                            connect(action, SIGNAL(triggered()), this, SLOT(onIActionChosen_SetText()));
                        }
                        controlSubMenu->addActions(actionsForControl);
                        }
                        break;
                    default:
                        break;
                }
            }
        }
    }
}

QList<QAction*> IActionSetter::getActionsForControl(AbstractObj::ObjType objType)
{
    switch (objType)
    {
        case AbstractObj::CheckBox:
            return checkboxIActions;
            break;
        case AbstractObj::Radio:
            return radioIActions;
            break;
        case AbstractObj::Spinner:
            return spinnerIActions;
            break;
        case AbstractObj::Edit:
            return editIActions;
            break;
        case AbstractObj::VarList:
            return varlistIActions;
            break;
        case AbstractObj::VarName:
            return varnameIActions;
            break;
        case AbstractObj::File:
            return fileIActions;
            break;
        case AbstractObj::ListBox:
            return listboxIActions;
            break;
        case AbstractObj::ComboBox:
            return comboboxIActions;
            break;
        case AbstractObj::Button:
            return buttonIActions;
            break;
        case AbstractObj::Text:
            return textIActions;
            break;
        case AbstractObj::TextBox:
            return textboxIActions;
            break;
        case AbstractObj::GroupBox:
            return groupboxIActions;
            break;
        case AbstractObj::Frame:
            return frameIActions;
            break;
        case AbstractObj::Color:
            return colorIActions;
            break;
        case AbstractObj::Exp:
            return expIActions;
            break;
        default:
            return QList<QAction*>();
            break;
    }
}
