#include "mainwindow.h"
#include "pleasefindstatadialog.h"
#include "statapathdialog.h"

// About Visua dialog
#include "aboutvisuadialog.h"

// License window
#include "licensewindow.h"

// Disclaimer window
#include "disclaimerwindow.h"

// Credit
#include "creditswindow.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent)
{
    // Window icon
    setWindowIcon(QIcon(":/images/icons/visualogo16_16.png"));

    // Plugins
    loadPlugins();

    // Actions
    createActions();

    // Menus
    createMenus();

    // Title
    setWindowTitle("Visua 0.1 beta");

    // Document
    document = new Document;

    // Model
    model = new Model(this);
    model->setDocument(document);

    // Object list proxy model and its GUI
    objectListProxyModel = new ObjectListProxyModel(this);
    objectListProxyModel->setSourceModel(model);
    objectListView = new ObjectListView;
    objectListView->setModel(objectListProxyModel);
    objectListDockWidget = new ObjectListDockWidget(objectListView);
    addDockWidget(Qt::LeftDockWidgetArea, objectListDockWidget);

    // Object properties proxy model and its GUI
    objectPropertiesProxyModel = new ObjectPropertiesProxyModel(this);
    objectPropertiesProxyModel->setSourceModel(model);
    objectPropertiesView = new ObjectPropertiesView;
    objectPropertiesView->setModel(objectPropertiesProxyModel);
    objectPropertiesDockWidget = new ObjectPropertiesDockWidget(objectPropertiesView);
    addDockWidget(Qt::LeftDockWidgetArea, objectPropertiesDockWidget);

    // Code view
    codeView = new CodeView(document);
    codeViewDockWidget = new CodeViewDockWidget(codeView);
    addDockWidget(Qt::RightDockWidgetArea, codeViewDockWidget);

    // Object box
    objectBoxDockWidget = new ObjectBoxDockWidget;
    addDockWidget(Qt::RightDockWidgetArea, objectBoxDockWidget);

    // Canvas
    canvas = new Canvas(this);
    canvas->setModel(model);
    setCentralWidget(canvas);

    // Toolbar
    createToolBar();

    // Stata log view
    stataLogView = new StataLogView(this);

    // Help contents view
    helpContentsView = new HelpContentsView("index.htm", this);

    // Debugging
    debugReportWindow = new DebugReportWindow(document, this);

    // Connections
    connect(objectListView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            objectPropertiesProxyModel, SLOT(onSelectionChanged(QItemSelection,QItemSelection)));
    connect(objectListView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            objectPropertiesView,
            SLOT(onSelectionChangedInObjectListTreeView(QItemSelection,QItemSelection)));
    connect(model, SIGNAL(layoutChanged()), objectPropertiesProxyModel, SLOT(onMainModelWasResetted()));
    connect(model, SIGNAL(dataChanged(QModelIndex,QModelIndex)), this,
            SLOT(onDataChanged(QModelIndex,QModelIndex)));
    connect(model, SIGNAL(layoutChanged()), codeView, SLOT(updateView()));
    connect(model, SIGNAL(dataChanged(QModelIndex,QModelIndex)), codeView, SLOT(updateView()));
    connect(model, SIGNAL(modelWasResetted()), codeView, SLOT(updateView()));
    connect(objectListView, SIGNAL(controlAddedToDialog()), codeView, SLOT(updateView()));
    connect(objectListView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            canvas, SLOT(onSelectionChangedInObjectListView(QItemSelection,QItemSelection)));
    connect(model, SIGNAL(layoutAboutToBeChanged()),
            objectListProxyModel, SIGNAL(layoutAboutToBeChanged()));
    connect(model, SIGNAL(layoutChanged()),
            objectListProxyModel, SIGNAL(layoutChanged()));
    connect(model, SIGNAL(modelWasResetted()), objectListView, SLOT(reset()));
    connect(model, SIGNAL(modelWasResetted()), objectPropertiesProxyModel, SLOT(onMainModelWasResetted()));
    connect(objectListView, SIGNAL(itemMovedUp()), objectPropertiesProxyModel, SLOT(onItemMovedUp()));
    connect(objectListView, SIGNAL(itemMovedDown()), objectPropertiesProxyModel, SLOT(onItemMovedDown()));
    connect(canvas->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            objectListView, SLOT(onSelectionChangedInCanvas(QItemSelection,QItemSelection)));
    connect(objectListProxyModel, SIGNAL(newItemAppended(QModelIndex)),
            objectListView, SLOT(setCurrentIndex(QModelIndex)));
    connect(objectListProxyModel, SIGNAL(wasAttemptedToInsertProhibitedObjInDialog()),
            this, SLOT(onWasAttemptedToInsertProhibitedObjInDialog()));
    connect(objectPropertiesProxyModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
            this, SLOT(onDataChangedInObjectPropertiesProxyModel(QModelIndex,QModelIndex)));
    connect(objectListDockWidget, SIGNAL(visibilityChanged(bool)), viewObjectListAction, SLOT(setChecked(bool)));
    connect(objectPropertiesDockWidget, SIGNAL(visibilityChanged(bool)), viewObjectPropertiesAction, SLOT(setChecked(bool)));
    connect(codeViewDockWidget, SIGNAL(visibilityChanged(bool)), viewCodeAction, SLOT(setChecked(bool)));
    connect(objectBoxDockWidget, SIGNAL(visibilityChanged(bool)), viewObjectBoxAction, SLOT(setChecked(bool)));

    setCurrentFile("");
    readSettings();

}

void MainWindow::createActions()
{
    // New file
    newFileAction = new QAction("&New", this);
    newFileAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_N));
    newFileAction->setIcon(QIcon(":/images/icons/newdocument.png"));
    connect(newFileAction, SIGNAL(triggered()), this, SLOT(onNewFileActionTriggered()));

    // New from template
    createNewFromTemplateActions();

    // Open file
    openFileAction = new QAction("&Open", this);
    openFileAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_O));
    openFileAction->setIcon(QIcon(":/images/icons/opendocument.png"));
    connect(openFileAction, SIGNAL(triggered()), this, SLOT(onOpenFileActionTriggered()));

    // Save file
    saveFileAction = new QAction("&Save", this);
    saveFileAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_S));
    saveFileAction->setIcon(QIcon(":/images/icons/savedocument.png"));
    connect(saveFileAction, SIGNAL(triggered()), this, SLOT(onSaveFileActionTriggered()));

    // Save file as...
    saveFileAsAction = new QAction("Save &As...", this);
    saveFileAsAction->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_S));
    connect(saveFileAsAction, SIGNAL(triggered()), this, SLOT(saveAs()));

    // Export...
    exportAction = new QAction("Export...", this);
    exportAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_E));
    connect(exportAction, SIGNAL(triggered()), this, SLOT(onExportActionTriggered()));

    // Exit
    exitAction = new QAction("&Exit", this);
    exitAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_Q));
    exitAction->setIcon(QIcon(":/images/icons/exit.png"));
    connect(exitAction, SIGNAL(triggered()), qApp, SLOT(closeAllWindows()));

    // Test dialog in Stata
    testDialogInStataAction = new QAction("&Test dialog in Stata", this);
    testDialogInStataAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_T));
    testDialogInStataAction->setIcon(QIcon(":/images/icons/testdialoginstata.png"));
    connect(testDialogInStataAction, SIGNAL(triggered()),
            this, SLOT(onTestDialogInStataActionTriggered()));

    // View last stata log
    viewLastStataLogAction = new QAction("View &last Stata log file", this);
    viewLastStataLogAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_L));
    viewLastStataLogAction->setIcon(QIcon(":/images/icons/statalog.png"));
    connect(viewLastStataLogAction, SIGNAL(triggered()),
            this, SLOT(viewLastStataLog()));

    // Set Stata path
    stataPath = new QAction("Stata &path...", this);
    stataPath->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_P));
    connect(stataPath, SIGNAL(triggered()), this, SLOT(onStataPathTriggered()));

    // Debug
    debugAction = new QAction("&Debug report", this);
    debugAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_D));
    debugAction->setIcon(QIcon(":/images/icons/bug.png"));
    connect(debugAction, SIGNAL(triggered()), this, SLOT(onDebugActionTriggered()));

    // View last debug log
    viewLastDebugReportAction = new QAction("View last de&bug report", this);
    viewLastDebugReportAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_B));
    connect(viewLastDebugReportAction, SIGNAL(triggered()), this,
            SLOT(onViewLastDebugReportActionTriggered()));

    // Plugin actions
    for (int i=0; i<plugins.count(); i++)
    {
        QAction *pluginAction = new QAction(plugins.at(i)->pluginName(), this);
        pluginAction->setData(i);
        connect(pluginAction, SIGNAL(triggered()), this, SLOT(onPluginTriggered()));
        pluginActions.append(pluginAction);
    }

    // View object list
    viewObjectListAction = new QAction("Object list", this);
    viewObjectListAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_1));
    viewObjectListAction->setCheckable(true);
    viewObjectListAction->setChecked(true);
    connect(viewObjectListAction, SIGNAL(triggered(bool)), this,
            SLOT(onViewObjectListActionChecked(bool)));

    // View object properties
    viewObjectPropertiesAction = new QAction("Object properties", this);
    viewObjectPropertiesAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_2));
    viewObjectPropertiesAction->setCheckable(true);
    viewObjectPropertiesAction->setChecked(true);
    connect(viewObjectPropertiesAction, SIGNAL(triggered(bool)), this,
            SLOT(onViewObjectPropertiesActionChecked(bool)));

    // View code
    viewCodeAction = new QAction("Code", this);
    viewCodeAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_3));
    viewCodeAction->setCheckable(true);
    viewCodeAction->setChecked(true);
    connect(viewCodeAction, SIGNAL(triggered(bool)), this,
            SLOT(onViewCodeChecked(bool)));

    // View object box
    viewObjectBoxAction = new QAction("Object list", this);
    viewObjectBoxAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_4));
    viewObjectBoxAction->setCheckable(true);
    viewObjectBoxAction->setChecked(true);
    connect(viewObjectBoxAction, SIGNAL(triggered(bool)), this,
            SLOT(onViewObjectBoxActionChecked(bool)));

    // Help contents
    helpContentsAction = new QAction("&Contents", this);
    helpContentsAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_H));
    helpContentsAction->setIcon(QIcon(":/images/icons/help.png"));
    connect(helpContentsAction, SIGNAL(triggered()), this, SLOT(onHelpContentsActionTriggered()));

    // Help dialog programming
    helpDialogProgramming = new QAction("&Dialog programming", this);
    helpDialogProgramming->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_P));
    connect(helpDialogProgramming, SIGNAL(triggered()), this, SLOT(onHelpDialogProgrammingTriggered()));

    // Help dialog programming FAQs
    helpDialogProgrammingFAQsA1 = new QAction("Dialog programming (part 1) - Getting started", this);
    helpDialogProgrammingFAQsA1->setData(QVariant(QUrl("http://www.stata.com/support/faqs/programming/db-intro/dialog-programming/")));
    connect(helpDialogProgrammingFAQsA1, SIGNAL(triggered()),
            this, SLOT(onHelpDialogProgrammingFAQsTriggered()));
    helpDialogProgrammingFAQsA2 = new QAction("Dialog programming (part 2) - Adding features", this);
    helpDialogProgrammingFAQsA2->setData(QVariant(QUrl("http://stata.com/support/faqs/programming/db-intro/dialog-programming-adding-features/")));
    connect(helpDialogProgrammingFAQsA2, SIGNAL(triggered()),
            this, SLOT(onHelpDialogProgrammingFAQsTriggered()));
    helpDialogProgrammingFAQsA3 = new QAction("Dialog programming (part 3) - Using tabs", this);
    helpDialogProgrammingFAQsA3->setData(QVariant(QUrl("http://stata.com/support/faqs/programming/db-intro/dialog-programming-using-tabs/")));
    connect(helpDialogProgrammingFAQsA3, SIGNAL(triggered()),
            this, SLOT(onHelpDialogProgrammingFAQsTriggered()));
    helpDialogProgrammingFAQsA4 = new QAction("Dialog programming (part 4) - Using options", this);
    helpDialogProgrammingFAQsA4->setData(QVariant(QUrl("http://stata.com/support/faqs/programming/db-intro/dialog-programming-using-options/")));
    connect(helpDialogProgrammingFAQsA4, SIGNAL(triggered()),
            this, SLOT(onHelpDialogProgrammingFAQsTriggered()));
    helpDialogProgrammingFAQsB1 = new QAction("Sharing dialogs (part 1) - Basic terminology and overview", this);
    helpDialogProgrammingFAQsB1->setData(QVariant(QUrl("http://stata.com/support/faqs/programming/db-user/sharing-dialogs/")));
    connect(helpDialogProgrammingFAQsB1, SIGNAL(triggered()),
            this, SLOT(onHelpDialogProgrammingFAQsTriggered()));
    helpDialogProgrammingFAQsB2 = new QAction("Sharing dialogs (part 2) - Dialog-box programs", this);
    helpDialogProgrammingFAQsB2->setData(QVariant(QUrl("http://stata.com/support/faqs/programming/db-user/sharing-dialog-box-programs/")));
    connect(helpDialogProgrammingFAQsB2, SIGNAL(triggered()),
            this, SLOT(onHelpDialogProgrammingFAQsTriggered()));
    helpDialogProgrammingFAQsB3 = new QAction("Sharing dialogs (part 3) - Stata menus", this);
    helpDialogProgrammingFAQsB3->setData(QVariant(QUrl("http://stata.com/support/faqs/programming/db-user/sharing-dialog-menus/")));
    connect(helpDialogProgrammingFAQsB3, SIGNAL(triggered()),
            this, SLOT(onHelpDialogProgrammingFAQsTriggered()));
    helpDialogProgrammingFAQsB4 = new QAction("Sharing dialogs (part 4) - Submenus and dialogs", this);
    helpDialogProgrammingFAQsB4->setData(QVariant(QUrl("http://stata.com/support/faqs/programming/db-user/sharing-submenus-and-dialog-boxes/")));
    connect(helpDialogProgrammingFAQsB4, SIGNAL(triggered()),
            this, SLOT(onHelpDialogProgrammingFAQsTriggered()));
    helpDialogProgrammingFAQsC1 = new QAction("What can you tell me about using the .idlg files that are written by StataCorp and used in their dialogs?", this);
    helpDialogProgrammingFAQsC1->setData(QVariant(QUrl("http://stata.com/support/faqs/programming/idlg-files/")));
    connect(helpDialogProgrammingFAQsC1, SIGNAL(triggered()),
            this, SLOT(onHelpDialogProgrammingFAQsTriggered()));
    helpDialogProgrammingFAQsC2 = new QAction("How can I take care of prerequisites in dialog boxes?", this);
    helpDialogProgrammingFAQsC2->setData(QVariant(QUrl("http://stata.com/support/faqs/programming/predefinition-buttons-for-dialog-boxes/")));
    connect(helpDialogProgrammingFAQsC2, SIGNAL(triggered()),
            this, SLOT(onHelpDialogProgrammingFAQsTriggered()));
    helpDialogProgrammingFAQsC3 = new QAction("Can I use existing Stata .idlg files for my own graphics command dialog?", this);
    helpDialogProgrammingFAQsC3->setData(QVariant(QUrl("http://stata.com/support/faqs/programming/idlg-files-used-in-graphics-dialogs/")));
    connect(helpDialogProgrammingFAQsC3, SIGNAL(triggered()),
            this, SLOT(onHelpDialogProgrammingFAQsTriggered()));


    // License
    licenseAction = new QAction("License...", this);
    connect(licenseAction, SIGNAL(triggered()), this, SLOT(onLicenseActionTriggered()));

    // Disclaimer
    disclaimerAction = new QAction("Disclaimer...", this);
    connect(disclaimerAction, SIGNAL(triggered()), this, SLOT(onDisclaimerActionTriggered()));

    // About Visua
    aboutVisuaAction = new QAction("&About Visua...", this);
    aboutVisuaAction->setIcon(QIcon(":/images/icons/visualogo16_16.png"));
    aboutVisuaAction->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_A));
    connect(aboutVisuaAction, SIGNAL(triggered()), this, SLOT(onAboutVisuaActionTriggered()));

    // About Qt
    aboutQtAction = new QAction("&About Qt...", this);
    connect(aboutQtAction, SIGNAL(triggered()), qApp, SLOT(aboutQt()));

    // Credits
    creditsAction = new QAction("Credits...", this);
    connect(creditsAction, SIGNAL(triggered()), this, SLOT(onCreditsActionTriggered()));
}

void MainWindow::createMenus()
{
    // File menu
    fileMenu = menuBar()->addMenu("&File");
    fileMenu->addAction(newFileAction);
    QMenu *newFromTemplateMenu = new QMenu("New from template");
    newFromTemplateMenu->addActions(templateActions);
    if (templateActions.isEmpty())
    {
        QAction *noTemplateAction = new QAction("No template", this);
        noTemplateAction->setEnabled(false);
        newFromTemplateMenu->addAction(noTemplateAction);
    }
    fileMenu->addMenu(newFromTemplateMenu);
    fileMenu->addAction(openFileAction);
    fileMenu->addAction(saveFileAction);
    fileMenu->addAction(saveFileAsAction);
    fileMenu->addSeparator();
    fileMenu->addAction(exportAction);
    fileMenu->addSeparator();
    fileMenu->addAction(exitAction);

    // Stata menu
    stataMenu = menuBar()->addMenu("&Stata");
    stataMenu->addAction(testDialogInStataAction);
    stataMenu->addAction(viewLastStataLogAction);
    stataMenu->addAction(stataPath);

    // Debug menu
    debugMenu = menuBar()->addMenu("&Debug");
    debugMenu->addAction(debugAction);
    debugMenu->addAction(viewLastDebugReportAction);

    // Plugins menu
    pluginsMenu = menuBar()->addMenu("&Plugins");
    for (int i=0; i<pluginActions.count(); i++)
        pluginsMenu->addAction(pluginActions.at(i));
    if (plugins.isEmpty())
    {
        QAction *noPluginAction = new QAction("No plugin", this);
        noPluginAction->setEnabled(false);
        pluginsMenu->addAction(noPluginAction);
    }

    // View menu
    viewMenu = menuBar()->addMenu("&View");
    viewMenu->addAction(viewObjectListAction);
    viewMenu->addAction(viewObjectPropertiesAction);
    viewMenu->addAction(viewCodeAction);
    viewMenu->addAction(viewObjectBoxAction);

    // Help menu
    helpMenu = menuBar()->addMenu("&Help");
    helpMenu->addAction(helpContentsAction);
    helpMenu->addAction(helpDialogProgramming);
    QMenu *dialogFAQsMenu = new QMenu("Dialog FAQs");
    helpMenu->addMenu(dialogFAQsMenu);
    QMenu *dialogFAQsMenuA = new QMenu("How do I get started writing Stata dialogs?");
    dialogFAQsMenu->addMenu(dialogFAQsMenuA);
    dialogFAQsMenuA->addAction(helpDialogProgrammingFAQsA1);
    dialogFAQsMenuA->addAction(helpDialogProgrammingFAQsA2);
    dialogFAQsMenuA->addAction(helpDialogProgrammingFAQsA3);
    dialogFAQsMenuA->addAction(helpDialogProgrammingFAQsA4);
    QMenu *dialogFAQsMenuB = new QMenu("How do I share user-written dialogs and Stata menus");
    dialogFAQsMenu->addMenu(dialogFAQsMenuB);
    dialogFAQsMenuB->addAction(helpDialogProgrammingFAQsB1);
    dialogFAQsMenuB->addAction(helpDialogProgrammingFAQsB2);
    dialogFAQsMenuB->addAction(helpDialogProgrammingFAQsB3);
    dialogFAQsMenuB->addAction(helpDialogProgrammingFAQsB4);
    QMenu *dialogFAQsMenuC = new QMenu("Advanced dialog topics");
    dialogFAQsMenu->addMenu(dialogFAQsMenuC);
    dialogFAQsMenuC->addAction(helpDialogProgrammingFAQsC1);
    dialogFAQsMenuC->addAction(helpDialogProgrammingFAQsC2);
    dialogFAQsMenuC->addAction(helpDialogProgrammingFAQsC3);
    helpMenu->addSeparator();
    helpMenu->addAction(licenseAction);
    helpMenu->addAction(disclaimerAction);
    helpMenu->addSeparator();
    helpMenu->addAction(aboutVisuaAction);
    helpMenu->addAction(aboutQtAction);
    helpMenu->addSeparator();
    helpMenu->addAction(creditsAction);
}

void MainWindow::createToolBar()
{
    toolBar = addToolBar("Toolbar");
    toolBar->addAction(newFileAction);
    toolBar->addAction(openFileAction);
    toolBar->addAction(saveFileAction);
    toolBar->addSeparator();
    toolBar->addAction(testDialogInStataAction);
    toolBar->addAction(viewLastStataLogAction);
    toolBar->addSeparator();
    toolBar->addAction(debugAction);
    toolBar->addSeparator();
    toolBar->addAction(helpContentsAction);
}

void MainWindow::onNewFileActionTriggered()
{
    if (okToContinue())
    {
        model->clear();
        canvas->resetRowOfSelectedDialog();
        setCurrentFile("");
    }
}

void MainWindow::onOpenFileActionTriggered()
{
    if (okToContinue())
    {
        QDir dir(lastDirForOpen);
        if (!dir.exists())
            lastDirForOpen = ".";

        QString fileName = QFileDialog::getOpenFileName(
                    this,
                    "Open Visua file",
                    lastDirForOpen,
                    "Visua files (*.vis)");

        if (!fileName.isEmpty())
        {
            lastDirForOpen = QFileInfo(fileName).absolutePath();
            loadVisuaFile(fileName);
            canvas->resetRowOfSelectedDialog();
        }
    }
}

bool MainWindow::onSaveFileActionTriggered()
{
    if (currentFile.isEmpty())
        return saveAs();
    else
        return saveVisuaFile(currentFile);
}

QString MainWindow::strippedFileName(const QString &fullFileName)
{
    return QFileInfo(fullFileName).fileName();
}

void MainWindow::setCurrentFile(const QString &fileName)
{
    currentFile = fileName;
    setWindowModified(false);

    QString shownFileName;
    if (currentFile.isEmpty())
        shownFileName = "Untitled";
    else
        shownFileName = strippedFileName(currentFile);

    setWindowTitle(QString("%1[*] - Visua").arg(shownFileName));
}

bool MainWindow::okToContinue()
{
    if (isWindowModified())
    {
        int resultFromMessageBox = QMessageBox::warning(
            this,
            "Visua",
            "The document has been modified.\nDo you want to save the changes?",
            QMessageBox::Yes | QMessageBox::Default,
            QMessageBox::No,
            QMessageBox::Cancel | QMessageBox::Escape);
        if (resultFromMessageBox == QMessageBox::Yes)
            return onSaveFileActionTriggered();
        else if (resultFromMessageBox == QMessageBox::Cancel)
            return false;
        else
            return true;
    }
    else
        return true;
}

bool MainWindow::loadVisuaFile(const QString &fileName)
{
    if (model->loadVisuaFile(fileName) == false)
    {
        QMessageBox::critical(
                    this,
                    "Visua",
                    "File error. The issue is one of the following:\n1) Error opening the file\n2) The file is not a Visua file\n3) The file is a Visua file but its version is not supported by this version of Visua");
        return false;
    }
    else
    {
        setCurrentFile(fileName);
        codeView->update();
        return true;
    }
}

bool MainWindow::saveVisuaFile(const QString &fileName)
{
    if (document->writeVisuaFile(fileName) == false)
        return false;
    else
    {
        setCurrentFile(fileName);
        return true;
    }
}

bool MainWindow::saveAs()
{
    QDir dir(lastDirForSaveAs);
    if (!dir.exists())
        lastDirForSaveAs = ".";

    QString fileName = QFileDialog::getSaveFileName(
                this,
                "Save Visua file",
                lastDirForSaveAs,
                "Visua files (*.vis)");

    if (fileName.isEmpty())
        return false;
    else
    {
        lastDirForSaveAs = QFileInfo(fileName).absolutePath();
        return saveVisuaFile(fileName);
    }
}

void MainWindow::onExportActionTriggered()
{
    QDir dir(lastDirForExport);
    if (!dir.exists())
        lastDirForExport = ".";

    QString fileName = QFileDialog::getSaveFileName(
                this,
                "Export to dialog resource",
                lastDirForExport,
                "Dialog resource (*.dlg);;Include dialog resource (*.idlg)");

    if (!fileName.isEmpty())
    {
        lastDirForExport = QFileInfo(fileName).absolutePath();
        document->writeStataDlgCodeOnFile(fileName);
    }
}

void MainWindow::onDataChanged(QModelIndex, QModelIndex)
{
    setWindowModified(true);
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    if (okToContinue())
    {
        writeSettings();
        event->accept();
    }
    else
        event->ignore();
}

void MainWindow::onTestDialogInStataActionTriggered()
{
    if (stataExecutablePath.isEmpty())
    {
        PleaseFindStataDialog pleaseFindStataDialog(this);
        int ret = pleaseFindStataDialog.exec();
        if (ret == QDialog::Accepted)
            stataExecutablePath = pleaseFindStataDialog.getStataLocation();
        else
            return;
    }

    // "Visuatest" dir
    QDir visuaTestDir(QApplication::applicationDirPath());
    bool visuaTestDirWasFound = visuaTestDir.cd("Visuatest");
    if (!visuaTestDirWasFound)
    {
        QMessageBox::critical(this,
                              "Visua",
                              "The \"Visuatest\" folder was not found");
        return;
    }
    QString visuaTestAbsolutePath = visuaTestDir.absolutePath();

    // Create do file
    QString visuaTestDoFile = visuaTestAbsolutePath + QString("/visuatest.do");
    QFile fileDo(visuaTestDoFile);
    if (!fileDo.open(QIODevice::WriteOnly | QIODevice::Text))
    {
        QMessageBox::critical(this,
                              "Visua",
                              "Error writing \"visuatest.do\"");
        return;
    }
    QTextStream fileDoTextStream(&fileDo);
    fileDoTextStream << "cd " << "\"" << visuaTestAbsolutePath << "\"" << "\n";
    fileDoTextStream << "db visuatest" << "\n";
    fileDo.close();

    // Create dlg file
    QString visuaTestDlgFile = visuaTestAbsolutePath + QString("/visuatest.dlg");
    document->writeStataDlgCodeOnFile(visuaTestDlgFile);

    // Temporarily set the "Visuatest" folder as current directory
    QDir::setCurrent(visuaTestDir.absolutePath());

    // Launch Stata
    QStringList arguments;
    arguments << "/b" << "do" << visuaTestDoFile;
    QProcess *myProcess = new QProcess(this);
    connect(myProcess, SIGNAL(error(QProcess::ProcessError)),
            this, SLOT(onErrorInStataProcess(QProcess::ProcessError)));
    myProcess->start(stataExecutablePath, arguments);

    // Set the application folder as current directory
    QDir::setCurrent(QApplication::applicationDirPath());
}

void MainWindow::prova()
{
    QModelIndex index = objectListProxyModel->index(2, 1);
    objectListView->update(index);
}

void MainWindow::writeSettings()
{
    QDir settingsDir(QApplication::applicationDirPath());
    bool settingsDirWasFound = settingsDir.cd("Settings");
    if (!settingsDirWasFound)
    {
        QMessageBox::critical(this,
                              "Visua",
                              "The \"Settings\" folder was not found");
        return;
    }

    QString visuaSettingsFile = settingsDir.filePath("visuasettings.ini");

    QSettings settings(visuaSettingsFile, QSettings::IniFormat);

    // Stata path
    settings.beginGroup("stata");
    settings.setValue("statapath", stataExecutablePath);
    settings.endGroup();

    // Main window
    settings.beginGroup("mainwindow");
    settings.setValue("fullscreen", isMaximized());
    settings.setValue("geometry", geometry());
    settings.endGroup();

    // Stata log view
    settings.beginGroup("statalogwindow");
    settings.setValue("geometry", stataLogView->geometry());
    settings.endGroup();

    // Debug report window
    settings.beginGroup("debugreportwindow");
    settings.setValue("geometry", debugReportWindow->geometry());
    settings.setValue("splitterstate", debugReportWindow->splitterSaveState());
    settings.endGroup();

    // Help contents
    settings.beginGroup("helpcontents");
    settings.setValue("geometry", helpContentsView->geometry());
    settings.endGroup();

    // Last directories
    settings.beginGroup("lastdirectories");
    settings.setValue("lastdirforopen", lastDirForOpen);
    settings.setValue("lastdirforsaveas", lastDirForSaveAs);
    settings.setValue("lastdirforexport", lastDirForExport);
    settings.endGroup();
}

void MainWindow::readSettings()
{
    QDir settingsDir(QApplication::applicationDirPath());
    bool settingsDirWasFound = settingsDir.cd("Settings");
    if (!settingsDirWasFound)
    {
        QMessageBox::critical(this,
                              "Visua",
                              "The \"Settings\" folder was not found");
        return;
    }

    QString visuaSettingsFile = settingsDir.filePath("visuasettings.ini");

    QSettings settings(visuaSettingsFile, QSettings::IniFormat);

    // Stata path
    settings.beginGroup("stata");
    stataExecutablePath = settings.value("statapath", "").toString();
    settings.endGroup();

    // Main window
    {
    settings.beginGroup("mainwindow");
    bool fullscreen = settings.value("fullscreen", true).toBool();
    if (fullscreen)
        showMaximized();
    else
        setGeometry(settings.value("geometry", QRect(30, 30, 800, 600)).toRect());
    settings.endGroup();
    }

    // Stata log view
    settings.beginGroup("statalogwindow");
    stataLogView->setGeometry(settings.value("geometry", QRect(70, 70, 600, 400)).toRect());
    settings.endGroup();

    // Debug report window
    settings.beginGroup("debugreportwindow");
    debugReportWindow->setGeometry(settings.value("geometry", QRect(50, 50, 400, 300)).toRect());
    debugReportWindow->restoreSplitterState(settings.value("splitterstate").toByteArray());
    settings.endGroup();

    // Help contents
    settings.beginGroup("helpcontents");
    helpContentsView->setGeometry(settings.value("geometry", QRect(21, 21, 600, 400)).toRect());
    settings.endGroup();

    // Last directories
    settings.beginGroup("lastdirectories");
    lastDirForOpen = settings.value("lastdirforopen", ".").toString();
    lastDirForSaveAs = settings.value("lastdirforsaveas", ".").toString();
    lastDirForExport = settings.value("lastdirforexport", ".").toString();
    settings.endGroup();
}

void MainWindow::viewLastStataLog()
{
    if (stataLogView->loadLastLogFile())
        stataLogView->showNormal();
    else
        QMessageBox::warning(
            this,
            "Visua",
            "Last log file was not found or it was not possible to open the log file",
            QMessageBox::Ok);
}

void MainWindow::onStataPathTriggered()
{
    StataPathDialog stataPathDialog(stataExecutablePath, this);
    int ret = stataPathDialog.exec();
    if (ret == QDialog::Accepted)
        stataExecutablePath = stataPathDialog.getStataPath();
    else
        return;
}

void MainWindow::onViewObjectListActionChecked(bool checked)
{
    if (checked)
        objectListDockWidget->show();
    else
        objectListDockWidget->hide();
}

void MainWindow::onViewObjectPropertiesActionChecked(bool checked)
{
    if (checked)
        objectPropertiesDockWidget->show();
    else
        objectPropertiesDockWidget->hide();
}


void MainWindow::onViewCodeChecked(bool checked)
{
    if (checked)
        codeViewDockWidget->show();
    else
        codeViewDockWidget->hide();
}

void MainWindow::onViewObjectBoxActionChecked(bool checked)
{
    if (checked)
        objectBoxDockWidget->show();
    else
        objectBoxDockWidget->hide();
}

void MainWindow::onHelpContentsActionTriggered()
{
    helpContentsView->setSource("index.htm");
    helpContentsView->clearHistory();
    helpContentsView->showNormal();
}

void MainWindow::onHelpDialogProgrammingTriggered()
{
    if (stataExecutablePath.isEmpty())
    {
        PleaseFindStataDialog pleaseFindStataDialog(this);
        int ret = pleaseFindStataDialog.exec();
        if (ret == QDialog::Accepted)
            stataExecutablePath = pleaseFindStataDialog.getStataLocation();
        else
            return;
    }

    // Launch "help dialog" in Stata: "C:\Stata10\wsestata.exe help dialog"
    QStringList arguments;
    arguments << "help" << "dialog";
    QProcess *myProcess = new QProcess(this);
    connect(myProcess, SIGNAL(error(QProcess::ProcessError)),
            this, SLOT(onErrorInStataProcess(QProcess::ProcessError)));
    myProcess->start(stataExecutablePath, arguments);
}

void MainWindow::onHelpDialogProgrammingFAQsTriggered()
{
    QAction *action = qobject_cast<QAction*>(sender());
    QUrl url = action->data().toUrl();
    if (!QDesktopServices::openUrl(url))
    {
        QMessageBox::warning(
                    this,
                    "Visua",
                    QString("Error opening the following url:\n%1").arg(url.toString()));
    }
}

void MainWindow::onAboutVisuaActionTriggered()
{
    AboutVisuaDialog aboutVisuaDialog(this);
    aboutVisuaDialog.exec();
}

void MainWindow::onErrorInStataProcess(QProcess::ProcessError processError)
{
    QMessageBox::warning(
        this,
        "Visua",
        "Error in executing Stata.\nPlease check the Stata path",
        QMessageBox::Ok);
}

void MainWindow::loadPlugins()
{
    QDir pluginDir(QApplication::applicationDirPath());

    // Check if the "plugins" folder exists
    if (!pluginDir.cd("plugins"))
        return;

    foreach (QString fileName, pluginDir.entryList(QDir::Files))
    {
        QPluginLoader pluginLoader(pluginDir.absoluteFilePath(fileName));
        VisuaPluginInterface *plugin = qobject_cast<VisuaPluginInterface*>(pluginLoader.instance());
        if (plugin)
            plugins.append(plugin);
    }
}

void MainWindow::onPluginTriggered()
{
    QAction *action = qobject_cast<QAction *>(sender());
    int pluginId = action->data().toInt();
    VisuaPluginInterface *plugin = plugins.at(pluginId);
    QModelIndexList selectedIndexes = objectListView->selectionModel()->selectedRows(0);
    QString visuaCode;

    // Check behaviour
    switch (plugin->behaviour())
    {
        case VisuaPluginInterface::CreateNewVisuaDocument:
        case VisuaPluginInterface::AppendAlways:
            // Do nothing
            break;
        case VisuaPluginInterface::AppendToDialogOnly:
            {
            if (selectedIndexes.count() > 1)
            {
                QMessageBox::warning(
                            this,
                            "Visua",
                            QString("The plugin \"%1\" requires that only one object is selected")
                            .arg(plugin->pluginName()));
                return;
            }
            if (selectedIndexes.isEmpty())
            {
                QMessageBox::warning(
                            this,
                            "Visua",
                            QString("The plugin \"%1\" requires that a dialog is selected")
                            .arg(plugin->pluginName()));
                return;
            }
            if (selectedIndexes.count() == 1)
            {
                QModelIndex index = selectedIndexes.at(0);
                AbstractObj *obj = static_cast<AbstractObj*>(index.internalPointer());
                if (obj->objType() != AbstractObj::Dialog)
                {
                    QMessageBox::warning(
                                this,
                                "Visua",
                                QString("The plugin \"%1\" requires that a dialog is selected")
                                .arg(plugin->pluginName()));
                    return;
                }
            }
            }
            break;
        case VisuaPluginInterface::AppendOutsideDialogOnly:
            if (selectedIndexes.count() > 1)
            {
                QMessageBox::warning(
                            this,
                            "Visua",
                            QString("The plugin \"%1\" requires that no more than one object is selected")
                            .arg(plugin->pluginName()));
                return;
            }
            if (selectedIndexes.count() == 1)
            {
                QModelIndex index = selectedIndexes.at(0);
                QModelIndex parentIndex = index.parent();
                if (parentIndex != QModelIndex())
                {
                    QMessageBox::warning(
                                this,
                                "Visua",
                                QString("The plugin \"%1\" can't insert objects inside a dialog. Please select an object outside any dialog")
                                .arg(plugin->pluginName()));
                    return;
                }
            }
            break;
        default:
            QMessageBox::critical(
                        this,
                        "Visua",
                        QString("The behaviour of the plugin \"%1\" is not supported by this version of Visua")
                        .arg(plugin->pluginName()));
            return;
    }

    // Execute the plugin
    if (plugin->isDialog())
    {
        int dialogReturn = plugin->dialogExec(this);
        if (dialogReturn == QDialog::Accepted)
            visuaCode = plugin->visuaCode();
        else
            return;
    }
    else
        visuaCode = plugin->visuaCode();

    // Create list of objects
    QList<AbstractObj*> objList = document->objListFromVisuaCode(visuaCode);

    // Update Visua document
    switch (plugin->behaviour())
    {
        case VisuaPluginInterface::CreateNewVisuaDocument:
            {
            int choice = QMessageBox::warning(
                    this,
                    "Visua",
                    QString("The plugin\"%1\" is trying to create a new Visua document. The current Visua document will be lost. Do you want to continue?").arg(plugin->pluginName()),
                    QMessageBox::Yes, QMessageBox::No);
            if (choice == QMessageBox::Yes)
            {
                model->clear();
                setCurrentFile("");
                model->appendObjList(objList, QModelIndex());
            }
            }
            break;
        case VisuaPluginInterface::AppendAlways:
            if (selectedIndexes.isEmpty())
                model->appendObjList(objList, QModelIndex());
            else
            {
                QModelIndex index = selectedIndexes.at(0);
                AbstractObj *obj = static_cast<AbstractObj*>(index.internalPointer());
                if (obj->objType() == AbstractObj::Dialog)
                {
                    int choice = QMessageBox::information(
                                this,
                                "Visua",
                                "Do you want to append the Visua code to the selected dialog?",
                                QMessageBox::Yes, QMessageBox::No);
                    if (choice == QMessageBox::Yes)
                        model->appendObjList(objList, index);
                    else
                    {
                        QModelIndex parentIndex = index.parent();
                        model->insertObjList(objList, index.row() + 1, parentIndex);
                    }
                }
                else
                {
                    QModelIndex parentIndex = index.parent();
                    model->insertObjList(objList, index.row() + 1, parentIndex);
                }
            }
            break;
        case VisuaPluginInterface::AppendToDialogOnly:
            {
            QModelIndex index = selectedIndexes.at(0); // Index is surely a dialog (it was previously checked)
            model->appendObjList(objList, index);
            }
            break;
        case VisuaPluginInterface::AppendOutsideDialogOnly:
            if (selectedIndexes.isEmpty())
                model->appendObjList(objList, QModelIndex());
            else
            {
                QModelIndex index = selectedIndexes.at(0);
                QModelIndex parentIndex = index.parent(); // Parent index should be not a dialog
                model->insertObjList(objList, index.row() + 1, parentIndex);
            }
            break;
        default:
            break;
    }
}

void MainWindow::createNewFromTemplateActions()
{
    QDir templatesDir(QApplication::applicationDirPath());

    // Check if the "templates" folder exists
    if (!templatesDir.cd("templates"))
        return;

    int i = 0;
    foreach (QString fileName, templatesDir.entryList(QDir::Files))
    {
        QString templateFilePath = templatesDir.absoluteFilePath(fileName);
        QFileInfo templateFileInfo(templateFilePath);
        QString templateName = templateFileInfo.baseName();
        QAction *templateAction = new QAction(templateName, this);
        templateAction->setData(templateFilePath);
        connect(templateAction, SIGNAL(triggered()), this, SLOT(onNewFromTemplateTriggered()));
        templateActions.append(templateAction);
        i++;
    }
}

void MainWindow::onNewFromTemplateTriggered()
{
    QAction *action = qobject_cast<QAction *>(sender());
    QString fileName = action->data().toString();

    if (okToContinue())
    {
        loadVisuaFile(fileName);
        canvas->resetRowOfSelectedDialog();
        setCurrentFile("");
    }
}

void MainWindow::onWasAttemptedToInsertProhibitedObjInDialog()
{
    QMessageBox::warning(this, "Visua", "You can't add this object to a dialog");
}

void MainWindow::onDataChangedInObjectPropertiesProxyModel(const QModelIndex &topLeft,
                                                           const QModelIndex &bottomRight)
{
    Q_UNUSED(bottomRight)
    AbstractObj *obj = static_cast<AbstractObj*>(topLeft.internalPointer());
    if (obj->hasName() && topLeft.row() == 0 && topLeft.column() == 1)
    {
        QModelIndex sourceIndex = objectPropertiesProxyModel->mapToSource(topLeft);
        QModelIndex objectListIndex = objectListProxyModel->mapFromSource(sourceIndex);
        objectListView->update(objectListIndex);
    }
}

void MainWindow::onDebugActionTriggered()
{
    debugReportWindow->updateReport();
    debugReportWindow->showNormal();
}

void MainWindow::onViewLastDebugReportActionTriggered()
{
    debugReportWindow->showNormal();
}

void MainWindow::onLicenseActionTriggered()
{
    LicenseWindow licenseWindow(this);
    licenseWindow.exec();
}

void MainWindow::onDisclaimerActionTriggered()
{
    DisclaimerWindow disclaimerWindow(this);
    disclaimerWindow.exec();
}

void MainWindow::onCreditsActionTriggered()
{
    CreditsWindow creditsWindow(this);
    creditsWindow.exec();
}
