#include "model.h"
#include "objtypemimedata.h"

#include <QDebug>

Model::Model(QObject *parent) :
    QAbstractItemModel(parent)
{
    m_document = 0;
}

void Model::setDocument(Document *document)
{
    m_document = document;
    reset();
    emit modelWasResetted();
}

QModelIndex Model::index(int row, int column, const QModelIndex &parent) const
{
    if (!hasIndex(row, column, parent))
        return QModelIndex();

    if (!parent.isValid())
    {
        AbstractObj *obj = m_document->objAt(row);
        return createIndex(row, column, obj);
    }
    else
    {
        AbstractObj *parentObj = static_cast<DialogObj*>(parent.internalPointer());
        AbstractObj *childObj = parentObj->objAt(row);
        return createIndex(row, column, childObj);
    }
}

QModelIndex Model::parent(const QModelIndex &index) const
{
    if (!index.isValid())
        return QModelIndex();

    AbstractObj *childObj = static_cast<AbstractObj*>(index.internalPointer());
    AbstractObj *parentObj = childObj->parentObj();
    if (parentObj == m_document->root())
        return QModelIndex();
    else
        return createIndex(parentObj->row(), 0, parentObj);
}

int Model::rowCount(const QModelIndex &parent) const
{
    // Only indexes at column 0 have child indexes
    if (parent.column() > 0)
        return 0;

    if (!parent.isValid())
        return m_document->objCount();
    else
    {
        AbstractObj *obj = static_cast<AbstractObj*>(parent.internalPointer());
        return obj->objCount();
    }
}

int Model::columnCount(const QModelIndex &parent) const
{
    return 32; // It's an arbitrary number
}

QVariant Model::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
        return QVariant();

    AbstractObj *obj = static_cast<AbstractObj*>(index.internalPointer());
    switch (role)
    {
        case Qt::DisplayRole:
            return obj->displayRoleDataAt(index.column());
            break;
        case Qt::EditRole:
            return obj->editRoleDataAt(index.column());
            break;
        default:
            return QVariant();
            break;
    }
}

bool Model::setData(const QModelIndex &index, const QVariant &value, int role)
{
    if (!index.isValid())
        return false;

    switch (role)
    {
        case Qt::EditRole:
            {
            AbstractObj* obj = static_cast<AbstractObj*>(index.internalPointer());
            if (obj->setEditRoleDataForPropertyAt(index.column() - 1, value))
            {
                emit dataChanged(index, index);
                return true;
            }
            else
                return false;
            }
            break;
        default:
            return false;
            break;
    }
}

bool Model::removeRows(int row, int count, const QModelIndex &parent)
{
    emit layoutAboutToBeChanged();
    beginRemoveRows(parent, row, row + count - 1);
    for (int i = 0; i < count; i++)
    {
        if (!parent.isValid())
        {
            m_document->deleteObjAt(row);
        }
        else
        {
            AbstractObj *parentObj = static_cast<AbstractObj*>(parent.internalPointer());
            parentObj->deleteObjAt(row);
        }
    }
    endRemoveRows();
    emit layoutChanged();
//    reset(); // I don't know if it is useful
//    emit modelWasResetted();
    return true;
}

bool Model::loadVisuaFile(const QString &fileName)
{
    if (m_document->loadVisuaFile(fileName) == true)
    {
        reset();
        emit modelWasResetted();
        return true;
    }
    else
        return false;
}

void Model::clear()
{
    m_document->clear();
    reset();
    emit modelWasResetted();
}

bool Model::moveItemUp(const QModelIndex &parent, int row)
{
    beginMoveRows(parent, row, row, parent, row-1);
    if (parent == QModelIndex())
        m_document->moveObjUp(-1, row);
    else
        m_document->moveObjUp(parent.row(), row);
    endMoveRows();
    return true;
}

bool Model::moveItemDown(const QModelIndex &parent, int row)
{
    beginMoveRows(parent, row, row, parent, row+2);
    if (parent == QModelIndex())
        m_document->moveObjDown(-1, row);
    else
        m_document->moveObjDown(parent.row(), row);
    endMoveRows();
    return true;
}

AbstractObj *Model::newObjectOfType(AbstractObj::ObjType objType)
{
    switch (objType)
    {
        case AbstractObj::Version:
            return new VersionObj;
            break;
        case AbstractObj::Include:
            return new IncludeObj;
            break;
        case AbstractObj::Define:
            return new DefineObj;
            break;
        case AbstractObj::Position:
            return new PositionObj;
            break;
        case AbstractObj::List:
            return new ListObj;
            break;
        case AbstractObj::Dialog:
            return new DialogObj;
            break;
        case AbstractObj::CheckBox:
            return new CheckBoxObj;
            break;
        case AbstractObj::Radio:
            return new RadioObj;
            break;
        case AbstractObj::Spinner:
            return new SpinnerObj;
            break;
        case AbstractObj::Edit:
            return new EditObj;
            break;
        case AbstractObj::VarList:
            return new VarListObj;
            break;
        case AbstractObj::VarName:
            return new VarNameObj;
            break;
        case AbstractObj::File:
            return new FileObj;
            break;
        case AbstractObj::ListBox:
            return new ListBoxObj;
            break;
        case AbstractObj::ComboBox:
            return new ComboBoxObj;
            break;
        case AbstractObj::Button:
            return new ButtonObj;
            break;
        case AbstractObj::Text:
            return new TextObj;
            break;
        case AbstractObj::TextBox:
            return new TextBoxObj;
            break;
        case AbstractObj::GroupBox:
            return new GroupBoxObj;
            break;
        case AbstractObj::Frame:
            return new FrameObj;
            break;
        case AbstractObj::Color:
            return new ColorObj;
            break;
        case AbstractObj::Exp:
            return new ExpObj;
            break;
        case AbstractObj::Ok:
            return new OkObj;
            break;
        case AbstractObj::Submit:
            return new SubmitObj;
            break;
        case AbstractObj::Cancel:
            return new CancelObj;
            break;
        case AbstractObj::Copy:
            return new CopyObj;
            break;
        case AbstractObj::Help:
            return new HelpObj;
            break;
        case AbstractObj::Reset:
            return new ResetObj;
        case AbstractObj::Script:
            return new ScriptObj;
            break;
        case AbstractObj::Program:
            return new ProgramObj;
            break;
        default:
            return 0;
            break;
    }
}

QString Model::prefixNameForNewObj(AbstractObj::ObjType objType)
{
    // Note: Only objects with name are reported
    switch (objType)
    {
        case AbstractObj::Define:
            return "define";
            break;
        case AbstractObj::List:
            return "list";
            break;
        case AbstractObj::Dialog:
            return "dialog";
            break;
        case AbstractObj::CheckBox:
            return "checkbox";
            break;
        case AbstractObj::Radio:
            return "radio";
            break;
        case AbstractObj::Spinner:
            return "spinner";
            break;
        case AbstractObj::Edit:
            return "edit";
            break;
        case AbstractObj::VarList:
            return "varlist";
            break;
        case AbstractObj::VarName:
            return "varname";
            break;
        case AbstractObj::File:
            return "file";
            break;
        case AbstractObj::ListBox:
            return "listbox";
            break;
        case AbstractObj::ComboBox:
            return "combobox";
            break;
        case AbstractObj::Button:
            return "button";
            break;
        case AbstractObj::Text:
            return "text";
            break;
        case AbstractObj::TextBox:
            return "textbox";
            break;
        case AbstractObj::GroupBox:
            return "groupbox";
            break;
        case AbstractObj::Frame:
            return "frame";
            break;
        case AbstractObj::Color:
            return "color";
            break;
        case AbstractObj::Exp:
            return "exp";
            break;
        case AbstractObj::Ok:
            return "ok";
            break;
        case AbstractObj::Submit:
            return "submit";
            break;
        case AbstractObj::Cancel:
            return "cancel";
            break;
        case AbstractObj::Copy:
            return "copy";
            break;
        case AbstractObj::Help:
            return "help";
            break;
        case AbstractObj::Reset:
            return "reset";
            break;
        case AbstractObj::Program:
            return "program";
            break;
        case AbstractObj::Script:
            return "script";
            break;
        default:
            return QString();
            break;
    }
}

QString Model::newNameForObj(const QModelIndex &parent, AbstractObj::ObjType objType)
{
    QString prefix = prefixNameForNewObj(objType);
    int nObjInParent = rowCount(parent);
    QString objName;
    QList<int> objIdList;
    for (int i=0; i<nObjInParent; i++)
    {
        objName = index(i, 1, parent).data().toString();
        if (objName.startsWith(prefix))
        {
            bool conversionToIntIsOk = false;
            int number = objName.mid(prefix.length(), -1).toInt(&conversionToIntIsOk);
            if (conversionToIntIsOk)
                objIdList.append(number);
        }
    }

    int freeObjId = 1;
    while (objIdList.contains(freeObjId))
    {
        freeObjId++;
    }

    return QString("%1%2").arg(prefix).arg(freeObjId);
}

void Model::insertNewObj(AbstractObj::ObjType objType, const QModelIndex &parentIndex, int row)
{
    // Create new object
    AbstractObj *newObj = newObjectOfType(objType);
    if (newObj->hasName())
    {
        QString newName = newNameForObj(parentIndex, objType);
        newObj->propertyAt(0)->setValue(newName);
    }

    // Insert new object
    emit layoutAboutToBeChanged();
    beginInsertRows(parentIndex, row, row);
    if (parentIndex.isValid())
    {
        DialogObj *dialogObj = static_cast<DialogObj*>(parentIndex.internalPointer());
        if (newObj->isControlObj() && dialogObj->hasAtLeastOneChildControlObjBeforeRow(row))
        {
            AbstractControlObj *controlObj = static_cast<AbstractControlObj*>(newObj);
            controlObj->yProperty->setValue("+20");
        }
        dialogObj->insertObj(newObj, row);
    }
    else
    {
        if (newObj->isControlObj() && m_document->hasAtLeastOneChildControlObjBeforeRow(row))
        {
            AbstractControlObj *controlObj = static_cast<AbstractControlObj*>(newObj);
            controlObj->yProperty->setValue("+20");
        }
        m_document->insertObj(newObj, row);
    }
    endInsertRows();
    emit layoutChanged();
}

void Model::insertNewObj(AbstractObj::ObjType objType, const QModelIndex &parentIndex, int row,
                  int x, int y)
{
    // Create new object
    AbstractObj *newObj = newObjectOfType(objType);
    if (newObj->hasName())
    {
        QString newName = newNameForObj(parentIndex, objType);
        newObj->propertyAt(0)->setValue(newName);
        if (newObj->isControlObj())
        {
            AbstractControlObj *controlObj = static_cast<AbstractControlObj*>(newObj);
            controlObj->xProperty->setValue(QString("%1").arg(x));
            controlObj->yProperty->setValue(QString("%1").arg(y));
        }
    }

    // Insert new object
    emit layoutAboutToBeChanged();
    beginInsertRows(parentIndex, row, row);
    if (parentIndex.isValid())
    {
        DialogObj *dialogObj = static_cast<DialogObj*>(parentIndex.internalPointer());
        dialogObj->insertObj(newObj, row);
    }
    else
    {
        m_document->insertObj(newObj, row);
    }
    endInsertRows();
    emit layoutChanged();
}

void Model::appendNewObj(AbstractObj::ObjType objType, const QModelIndex &parentIndex)
{
    int nObjOfParent = rowCount(parentIndex);
    insertNewObj(objType, parentIndex, nObjOfParent);
}

void Model::appendNewObj(AbstractObj::ObjType objType, const QModelIndex &parentIndex, int x, int y)
{
    int nObjOfParent = rowCount(parentIndex);
    insertNewObj(objType, parentIndex, nObjOfParent, x, y);
}

Qt::ItemFlags Model::flags(const QModelIndex &index) const
{
    if (!index.isValid())
        return QAbstractItemModel::flags(index) | Qt::ItemIsDropEnabled;

    return QAbstractItemModel::flags(index) | Qt::ItemIsDropEnabled;
}

QStringList Model::mimeTypes() const
{
    QStringList types;
    types << "text/plain";
    return types;
}

void Model::insertObjList(QList<AbstractObj*> objList, int row, const QModelIndex &parentIndex)
{
    int nObj = objList.count();

    emit layoutAboutToBeChanged();
    beginInsertRows(parentIndex, row, row + nObj - 1);
    if (parentIndex.isValid())
    {
        DialogObj *dialogObj = static_cast<DialogObj*>(parentIndex.internalPointer());
        for (int i=0; i<nObj; i++)
            dialogObj->insertObj(objList.at(i), row + i);
    }
    else
    {
        for (int i=0; i<nObj; i++)
            m_document->insertObj(objList.at(i), row + i);
    }
    endInsertRows();
    emit layoutChanged();
}

void Model::appendObjList(QList<AbstractObj*> objList, const QModelIndex &parentIndex)
{
    int nObjOfParent = rowCount(parentIndex);
    insertObjList(objList, nObjOfParent, parentIndex);
}


