#include "objectboxview.h"
#include <QPushButton>
#include <QDrag>
#include <QApplication>
#include <QCursor>
#include <QDebug>
#include "abstractobj.h"

ObjectBoxView::ObjectBoxView(QWidget *parent) :
    QTreeView(parent)
{
    setDragEnabled(true);

    sourceModel = new ObjectBoxModel;
    proxyModel = new QSortFilterProxyModel(this);
    proxyModel->setSourceModel(sourceModel);
    proxyModel->setFilterKeyColumn(0);
    proxyModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
    proxyModel->setFilterWildcard("");
    proxyModel->setFilterRole(Qt::EditRole);
    setModel(proxyModel);
    setHeaderHidden(true);
    setIndentation(0);
    setRootIsDecorated(false);

    // Controls button
    QModelIndex controlsSourceIndex = sourceModel->index(0, 0);
    QPushButton *controlsButton = new QPushButton("Controls");
    connect(controlsButton, SIGNAL(clicked()), this, SLOT(onControlsButtonClicked()));
    setIndexWidget(proxyModel->mapFromSource(controlsSourceIndex), controlsButton);

    // Special buttons button
    QModelIndex specialButtonsSourceIndex = sourceModel->index(1, 0);
    QPushButton *specialButtonButton = new QPushButton("Special button");
    connect(specialButtonButton, SIGNAL(clicked()), this, SLOT(onSpecialButtonsButtonClicked()));
    setIndexWidget(proxyModel->mapFromSource(specialButtonsSourceIndex), specialButtonButton);

    // Miscellaneous button
    QModelIndex miscellaneousSourceIndex = sourceModel->index(2, 0);
    QPushButton *miscellaneousButton = new QPushButton("Miscellaneous");
    connect(miscellaneousButton, SIGNAL(clicked()), this, SLOT(onMiscellaneousButtonClicked()));
    setIndexWidget(proxyModel->mapFromSource(miscellaneousSourceIndex), miscellaneousButton);

    expandAll();
    controlsButtonIsExpanded = true;
    specialButtonsButtonIsExpanded = true;
    miscellaneousButtonIsExpanded = true;
}

void ObjectBoxView::setFilterString(const QString &string)
{
    proxyModel->setFilterWildcard(string);

    // Controls button
    QModelIndex controlsSourceIndex = sourceModel->index(0, 0);
    QModelIndex controlsProxyIndex = proxyModel->mapFromSource(controlsSourceIndex);
    if (proxyModel->hasIndex(controlsProxyIndex.row(), controlsProxyIndex.column()))
    {
        QPushButton *controlsButton = new QPushButton("Controls");
        connect(controlsButton, SIGNAL(clicked()), this, SLOT(onControlsButtonClicked()));
        setIndexWidget(controlsProxyIndex, controlsButton);
        if (controlsButtonIsExpanded)
            expand(controlsProxyIndex);
        else
            collapse(controlsProxyIndex);
    }

    // Special buttons button
    QModelIndex specialButtonsSourceIndex = sourceModel->index(1, 0);
    QModelIndex specialButtonsProxyIndex = proxyModel->mapFromSource(specialButtonsSourceIndex);
    if (proxyModel->hasIndex(specialButtonsProxyIndex.row(), specialButtonsProxyIndex.column()))
    {
        QPushButton *specialButtonButton = new QPushButton("Special button");
        connect(specialButtonButton, SIGNAL(clicked()), this, SLOT(onSpecialButtonsButtonClicked()));
        setIndexWidget(specialButtonsProxyIndex, specialButtonButton);
        if (specialButtonsButtonIsExpanded)
            expand(specialButtonsProxyIndex);
        else
            collapse(specialButtonsProxyIndex);
    }

    // Miscellaneous button
    QModelIndex miscellaneousSourceIndex = sourceModel->index(2, 0);
    QModelIndex miscellaneousProxyIndex = proxyModel->mapFromSource(miscellaneousSourceIndex);
    if (proxyModel->hasIndex(miscellaneousProxyIndex.row(), miscellaneousProxyIndex.column()))
    {
        QPushButton *miscellaneousButton = new QPushButton("Miscellaneous");
        connect(miscellaneousButton, SIGNAL(clicked()), this, SLOT(onMiscellaneousButtonClicked()));
        setIndexWidget(miscellaneousProxyIndex, miscellaneousButton);
        if (miscellaneousButtonIsExpanded)
            expand(miscellaneousProxyIndex);
        else
            collapse(miscellaneousProxyIndex);
    }
}

void ObjectBoxView::onControlsButtonClicked()
{
    QModelIndex index = proxyModel->index(0, 0);
    if (controlsButtonIsExpanded)
        collapse(index);
    else
        expand(index);
    controlsButtonIsExpanded = !controlsButtonIsExpanded;
}

void ObjectBoxView::onSpecialButtonsButtonClicked()
{
    QModelIndex index = proxyModel->index(1, 0);
    if (specialButtonsButtonIsExpanded)
        collapse(index);
    else
        expand(index);
    specialButtonsButtonIsExpanded = !specialButtonsButtonIsExpanded;
}

void ObjectBoxView::onMiscellaneousButtonClicked()
{
    QModelIndex index = proxyModel->index(2, 0);
    if (miscellaneousButtonIsExpanded)
        collapse(index);
    else
        expand(index);
    miscellaneousButtonIsExpanded = !miscellaneousButtonIsExpanded;
}

void ObjectBoxView::startDrag(Qt::DropActions supportedActions)
{
    Q_UNUSED(supportedActions)
    QDrag *drag = new QDrag(this);
    QModelIndexList indexes = selectedIndexes();
    drag->setMimeData(model()->mimeData(indexes));    

    QVariant objTypeVariantdata = selectedIndexes().at(0).data(Qt::UserRole);
    AbstractObj::ObjType objType = objTypeVariantdata.value<AbstractObj::ObjType>();
    switch (objType)
    {
        case AbstractObj::Version:
            drag->setPixmap(QPixmap(":images/icons/objicons/version_.png"));
            break;
        case AbstractObj::Include:
            drag->setPixmap(QPixmap(":images/icons/objicons/include_.png"));
            break;
        case AbstractObj::Define:
            drag->setPixmap(QPixmap(":images/icons/objicons/define_.png"));
            break;
        case AbstractObj::Position:
            drag->setPixmap(QPixmap(":images/icons/objicons/position_.png"));
            break;
        case AbstractObj::List:
            drag->setPixmap(QPixmap(":images/icons/objicons/list_.png"));
            break;
        case AbstractObj::Dialog:
            drag->setPixmap(QPixmap(":images/icons/objicons/dialog_.png"));
            break;
        case AbstractObj::CheckBox:
            drag->setPixmap(QPixmap(":images/icons/objicons/checkbox_.png"));
            break;
        case AbstractObj::Radio:
            drag->setPixmap(QPixmap(":images/icons/objicons/radio_.png"));
            break;
        case AbstractObj::Spinner:
            drag->setPixmap(QPixmap(":images/icons/objicons/spinner_.png"));
            break;
        case AbstractObj::Edit:
            drag->setPixmap(QPixmap(":images/icons/objicons/edit_.png"));
            break;
        case AbstractObj::VarList:
            drag->setPixmap(QPixmap(":images/icons/objicons/varlist_.png"));
            break;
        case AbstractObj::VarName:
            drag->setPixmap(QPixmap(":images/icons/objicons/varname_.png"));
            break;
        case AbstractObj::File:
            drag->setPixmap(QPixmap(":images/icons/objicons/file_.png"));
            break;
        case AbstractObj::ListBox:
            drag->setPixmap(QPixmap(":images/icons/objicons/listbox_.png"));
            break;
        case AbstractObj::ComboBox:
            drag->setPixmap(QPixmap(":images/icons/objicons/combobox_.png"));
            break;
        case AbstractObj::Button:
            drag->setPixmap(QPixmap(":images/icons/objicons/button_.png"));
            break;
        case AbstractObj::Text:
            drag->setPixmap(QPixmap(":images/icons/objicons/text_.png"));
            break;
        case AbstractObj::TextBox:
            drag->setPixmap(QPixmap(":images/icons/objicons/textbox_.png"));
            break;
        case AbstractObj::GroupBox:
            drag->setPixmap(QPixmap(":images/icons/objicons/groupbox_.png"));
            break;
        case AbstractObj::Frame:
            drag->setPixmap(QPixmap(":images/icons/objicons/frame_.png"));
            break;
        case AbstractObj::Color:
            drag->setPixmap(QPixmap(":images/icons/objicons/color_.png"));
            break;
        case AbstractObj::Exp:
            drag->setPixmap(QPixmap(":images/icons/objicons/exp_.png"));
            break;
        case AbstractObj::Ok:
            drag->setPixmap(QPixmap(":images/icons/objicons/ok_.png"));
            break;
        case AbstractObj::Submit:
            drag->setPixmap(QPixmap(":images/icons/objicons/submit_.png"));
            break;
        case AbstractObj::Cancel:
            drag->setPixmap(QPixmap(":images/icons/objicons/cancel_.png"));
            break;
        case AbstractObj::Copy:
            drag->setPixmap(QPixmap(":images/icons/objicons/copy_.png"));
            break;
        case AbstractObj::Help:
            drag->setPixmap(QPixmap(":images/icons/objicons/help_.png"));
            break;
        case AbstractObj::Reset:
            drag->setPixmap(QPixmap(":images/icons/objicons/reset_.png"));
            break;
        case AbstractObj::Script:
            drag->setPixmap(QPixmap(":images/icons/objicons/script_.png"));
            break;
        case AbstractObj::Program:
            drag->setPixmap(QPixmap(":images/icons/objicons/program_.png"));
            break;
        default:
            drag->setPixmap(QPixmap(":images/icons/test16x16.png"));
            break;
    }


    drag->start(Qt::CopyAction);
}

//void mouseMoveEvent(QMouseEvent *event)
//{

//}

